@@ -66,10 +66,10 @@ The FITS header structure: describes a whole header:
 ******************************************************************************/
 
 typedef struct
- {	char		name[16];
-	char		comment[80];
+ {	char		name[FITS_TAPE_CARDIMAGESIZE];
+	char		comment[FITS_TAPE_CARDIMAGESIZE];
+	char		vstr[FITS_TAPE_CARDIMAGESIZE];
 	int		vtype;
-	char		vstr[80];
 	int		vint;
 	double		vdouble;
  } fitsheader;
@@ -784,4 +784,4 @@ int		fits_cb_write (void *param,void *src,int length);
 #endif
 
 /*****************************************************************************/
-  
+                                       
