@@ -778,6 +778,10 @@ int get_planet_coords(double jdate, int n, double *rx, double *ry, double *rz)
      case 7: case 8: case 9:
 	mm=el-om-oo;	/* Default mean anomaly: M=L-\Omega-\omega=L-\varpi */
 	break;
+
+     default:
+	mm=0.0;
+	break;
     };
 
  e =solve_kepler_equ(mm*M_PI/180.0,ex);
