@@ -67,11 +67,88 @@ static fitsheader *fits_headerset_add(fitsheaderset *header)
  return(fhd);
 }
 
+static int fits_headerset_import_values(char *buff,int i0,char *comment,int *rtype,int *rvint,char *vstr,double *rvdouble)
+{
+ int	i,j,k,l,type,vint;
+ double	vdouble;
+
+ type=-1;
+ vint=0;
+ vdouble=0.0;
+
+ j=0;
+ for ( i=i0 ; i<FITS_TAPE_CARDIMAGESIZE ; i++ )
+  {	if ( buff[i]=='\'' )		j=!j;
+	if ( buff[i]=='/' && !j )	break;
+  }
+ if ( i<FITS_TAPE_CARDIMAGESIZE )
+  {	buff[i]=0;i++;
+	while ( buff[i]==32 )	i++;
+	strcpy(comment,buff+i);
+	while ( strlen(comment)>0 && comment[strlen(comment)-1]==32 )
+		comment[strlen(comment)-1]=0;
+  }
+
+ j=0;
+ for ( i=i0 ; i<strlen(buff) ; i++ )
+  {	if ( buff[i]=='-' || ('0'<=buff[i] && buff[i]<='9') || buff[i]=='.' || buff[i]=='+' )
+ 	 {	j=1;break;		}
+	else if ( buff[i] != 32 )
+ 	 {	j=2;break;		}
+  }
+
+ /** Numerical card image value:		**/
+ if ( j==1 )
+  {	for ( k=i,l=0 ; buff[k] ; k++ )
+	 {	if ( buff[k]=='.' )	l=1;	}
+	sscanf(buff+i,"%lg",&vdouble);
+	vint=(int)vdouble;
+	if ( (double)vint==vdouble && ! l )
+	 {	type=FITS_VINT;		}
+	else
+		type=FITS_VDOUBLE;
+  }
+
+ /** Otherwise, some kind of string or boolean:	**/
+ else if ( j==2 )
+  {	if ( buff[i]=='\'' )
+	 {	type=FITS_VSTR;
+		for ( j=i+1,k=0 ; buff[j] ; j++ )
+		 {	if ( buff[j]=='\'' && buff[j+1]=='\'' )
+			vstr[k]='\'',k++,j++;
+			else if ( buff[j]=='\'' )
+				break;
+			else
+				vstr[k]=buff[j],k++;
+		 }
+		vstr[k]=0;
+	 }
+	else
+	 {	for ( j=i,k=0 ; buff[j] ; j++ )
+			vstr[k]=buff[j],k++;
+		vstr[k]=0;
+		while ( k>0 && vstr[k-1]==32 )	k--,vstr[k]=0;
+		if ( strcmp(vstr,"T")==0 )
+			type=FITS_VBOOLEAN,vint=1;
+		else if ( strcmp(vstr,"F")==0 )
+			type=FITS_VBOOLEAN,vint=0;
+		else
+			type=FITS_VSTR;
+	 }
+   }
+
+ *rtype = type;
+ *rvint = vint;
+ *rvdouble = vdouble;
+
+ return(0);
+}
+
 int fits_headerset_read_cb(int (*cb_read)(void *,void *,int),void *param,fitsheaderset *header)
 {
  char		buff[2*FITS_TAPE_CARDIMAGESIZE];
  char		name[32],comment[2*FITS_TAPE_CARDIMAGESIZE];
- int		i,j,k,l,lin,type;
+ int		i,k,lin,type;
 
  int		vint;
  double		vdouble;
@@ -101,6 +178,8 @@ int fits_headerset_read_cb(int (*cb_read)(void *,void *,int),void *param,fitshea
 		continue;
 	 }
 
+	buff[FITS_TAPE_CARDIMAGESIZE]=0;
+
 	/** The card image is absolutely empty or contains the reserved	**/
 	/** keyword "END". Both cases indicate the end of the header.	**/
 	if ( k==FITS_TAPE_CARDIMAGESIZE || memcmp(buff,"END     ",8)==0 )
@@ -111,13 +190,14 @@ int fits_headerset_read_cb(int (*cb_read)(void *,void *,int),void *param,fitshea
 
 	/** Otherwise, its a normal card image with some context.	**/
 	else
-	 {	
+	 {	char	*p;
+
 		/** First, the keyword should be read...		**/
 		for ( i=0 ; i<8 && buff[i] != 32 ; i++ )
 		 {	name[i]=buff[i];			}
 
 		/** Reset the target storage variables...		**/ 
-		name[i]=0;j=0;type=0;vint=0;
+		name[i]=0;type=0;vint=0;vdouble=0.0;
 		buff[FITS_TAPE_CARDIMAGESIZE]=0;comment[0]=0;
 		
 		/** Standard commentatory keywords: COMMENT and HISTORY **/
@@ -127,66 +207,19 @@ int fits_headerset_read_cb(int (*cb_read)(void *,void *,int),void *param,fitshea
 			type=FITS_VCOMMENT;
 		 }
 
+		else if ( strcmp(name,"HIERARCH")==0 && buff[8]==' ' && (p=strchr(buff,'=')) != NULL && *(p-1)==' ' && *(p+1)==' ' )
+		 {	int	len,offset;
+			len=(p-buff)-1-9;
+			memcpy(name,buff+9,len);
+			name[len]=0;
+			offset=(p-buff)+2;
+			fits_headerset_import_values(buff,offset,comment,&type,&vint,vstr,&vdouble);
+		 }
+
 		/** The card image has an appropriate value field,	**/
 		/** indicated by the substring "= " at the position 8.	**/
 		else if ( memcmp(buff+8,"= ",2)==0 )	
-		 {	for ( i=10 ; i<FITS_TAPE_CARDIMAGESIZE ; i++ )
-			 {	if ( buff[i]=='\'' )		j=!j;
-				if ( buff[i]=='/' && !j )	break;
-			 }
-			if ( i<FITS_TAPE_CARDIMAGESIZE )
-			 {	buff[i]=0;i++;
-				while ( buff[i]==32 )	i++;
-				strcpy(comment,buff+i);
-				while ( strlen(comment)>0 && comment[strlen(comment)-1]==32 )
-					comment[strlen(comment)-1]=0;
-			 }
-		
-			j=0;
-			for ( i=10 ; i<strlen(buff) ; i++ )
-			 {	if ( buff[i]=='-' || ('0'<=buff[i] && buff[i]<='9') || buff[i]=='.' || buff[i]=='+' )
-				 {	j=1;break;		}
-				else if ( buff[i] != 32 )
-				 {	j=2;break;		}
-			 }
-			/** Numerical card image value:		**/
-			if ( j==1 )
-			 {	for ( k=i,l=0 ; buff[k] ; k++ )
-				 {	if ( buff[k]=='.' )	l=1;	}
-				sscanf(buff+i,"%lg",&vdouble);
-				vint=(int)vdouble;
-				if ( (double)vint==vdouble && ! l )
-				 {	type=FITS_VINT;		}
-				else
-					type=FITS_VDOUBLE;
-			 }
-			/** Otherwise, some kind of string or boolean:	**/
-			else if ( j==2 )
-			 {  if ( buff[i]=='\'' )
-			     {	type=FITS_VSTR;
-				for ( j=i+1,k=0 ; buff[j] ; j++ )
-				 {	if ( buff[j]=='\'' && buff[j+1]=='\'' )
-						vstr[k]='\'',k++,j++;
-					else if ( buff[j]=='\'' )
-						break;
-					else
-						vstr[k]=buff[j],k++;
-				 }
-				vstr[k]=0;
-			     }
-			    else
-			     {	for ( j=i,k=0 ; buff[j] ; j++ )
-					vstr[k]=buff[j],k++;
-				vstr[k]=0;
-				while ( k>0 && vstr[k-1]==32 )	k--,vstr[k]=0;
-				if ( strcmp(vstr,"T")==0 )
-					type=FITS_VBOOLEAN,vint=1;
-				else if ( strcmp(vstr,"F")==0 )
-					type=FITS_VBOOLEAN,vint=0;
-				else
-					type=FITS_VSTR;
-			     }
-			 }
+		 {	fits_headerset_import_values(buff,10,comment,&type,&vint,vstr,&vdouble);
 		 }
 
 		/** Any other case: the card image is a commentatory 	**/
@@ -196,6 +229,11 @@ int fits_headerset_read_cb(int (*cb_read)(void *,void *,int),void *param,fitshea
 			type=FITS_VCOMMENT;
 		 }
 
+		if ( type<0 )
+		 {	strcpy(vstr,buff+8);
+			type=FITS_VCOMMENT;
+		 }
+
 		fhd=fits_headerset_add(header);
 		
 		fhd->vtype=type;
@@ -231,12 +269,11 @@ int fits_headerset_read(FILE *fr,fitsheaderset *header)
 int fits_headerset_write_cb(int (*cb_write)(void *,void *,int),void *param,fitsheaderset *header)
 {
  char	buff[256],val[256];
- int	i,j,k,align,ln;
+ int	i,j,k,align,ln,is_hierarch;
 
  ln=0;
  for ( i=0 ; i<header->nhdr ; i++ )
-  {	strncpy(buff,header->hdrs[i].name,8);buff[8]=0;
-	while ( strlen(buff)<8 )	strcat(buff," ");
+  {	
 	align=0;
 	switch ( header->hdrs[i].vtype )
 	 {   case FITS_VINT:
@@ -271,40 +308,69 @@ int fits_headerset_write_cb(int (*cb_write)(void *,void *,int),void *param,fitsh
 		break;
 
 	 }
+
 	if ( ! align )	continue;
 
-	if ( align<0 )
-	 {	memset(buff,32,FITS_TAPE_CARDIMAGESIZE);
+	if ( 8<strlen(header->hdrs[i].name) )	
+		is_hierarch=1;
+	else
+		is_hierarch=0;
+
+	if ( ! is_hierarch )
+	 {
+		strncpy(buff,header->hdrs[i].name,8);buff[8]=0;
+		while ( strlen(buff)<8 )	strcat(buff," ");
+
+		if ( align<0 )
+		 {	memset(buff,32,FITS_TAPE_CARDIMAGESIZE);
+			buff[FITS_TAPE_CARDIMAGESIZE]=0;
+		 }
+
+		else if ( align==1 )
+		 {	strcat(buff,"= ");
+			strcat(buff,val);
+		 }
+		else if ( align==2 )
+		 {	strcat(buff,"= ");
+			while ( strlen(buff)+strlen(val)<30 )
+			 {	strcat(buff," ");		}
+			strcat(buff,val);
+		 }
+		else if ( align==3 )
+		 {	strcat(buff,val);			}
+
+		if ( header->hdrs[i].comment[0] && ( align==1 || align==2 ) )
+		 {	j=strlen(buff);
+			k=strlen(header->hdrs[i].comment);
+			while ( j<30 && j+k+3<FITS_TAPE_CARDIMAGESIZE )
+			 {	buff[j]=32,buff[j+1]=0;
+				j++;
+			 };
+			strcat(buff," / ");
+			strcat(buff,header->hdrs[i].comment);
+		 }
+
+		buff[FITS_TAPE_CARDIMAGESIZE]=0;
+		while ( strlen(buff)<FITS_TAPE_CARDIMAGESIZE )	strcat(buff," ");
+	 }
+
+	else
+	 {	int	n;
+
+		if ( header->hdrs[i].comment[0] )
+		 {	n=snprintf(buff,FITS_TAPE_CARDIMAGESIZE,
+				"HIERARCH %s = %s / %s",header->hdrs[i].name,val,header->hdrs[i].comment);
+		 }
+		else
+		 {	n=snprintf(buff,FITS_TAPE_CARDIMAGESIZE+1,
+				"HIERARCH %s = %s",header->hdrs[i].name,val);
+		 }
+		if ( n<FITS_TAPE_CARDIMAGESIZE )
+			memset(buff+n,32,FITS_TAPE_CARDIMAGESIZE-n);
+
 		buff[FITS_TAPE_CARDIMAGESIZE]=0;
 	 }
 
-	else if ( align==1 )
-	 {	strcat(buff,"= ");
-		strcat(buff,val);
-	 }
-	else if ( align==2 )
-	 {	strcat(buff,"= ");
-		while ( strlen(buff)+strlen(val)<30 )
-		 {	strcat(buff," ");		}
-		strcat(buff,val);
-	 }
-	else if ( align==3 )
-	 {	strcat(buff,val);			}
-
-	if ( header->hdrs[i].comment[0] && ( align==1 || align==2 ) )
-	 {	j=strlen(buff);
-		k=strlen(header->hdrs[i].comment);
-		while ( j<30 && j+k+3<FITS_TAPE_CARDIMAGESIZE )
-		 {	buff[j]=32,buff[j+1]=0;
-			j++;
-		 };
-		strcat(buff," / ");
-		strcat(buff,header->hdrs[i].comment);
-	 }
-
-	buff[FITS_TAPE_CARDIMAGESIZE]=0;
-	while ( strlen(buff)<FITS_TAPE_CARDIMAGESIZE )	strcat(buff," ");
-
 	cb_write(param,buff,FITS_TAPE_CARDIMAGESIZE);
 	ln++;
   }
@@ -641,4 +707,3 @@ int fits_headerset_is_extension(fitsheaderset *header)
 }
  
 /*****************************************************************************/
-                
