@@ -586,10 +586,17 @@ int fits_image_duplicate(fitsimage *ret,fitsimage *img,int flag)
 	memcpy(&ret->curr,&img->curr,sizeof(fitsscale));
 
 	/* this also sets ret->sx, ret->sy and ret->data properly */
+	/*
+	fprintf(stderr,"%d [%dx%d]\n",img->dim,img->naxis[0],img->naxis[1]);
+	*/
 	fits_image_alloc_gen(ret,img->dim,img->naxis);
 	ntot=fits_image_total_pixels(img->dim,img->naxis);
 	idata=fits_image_first_pixel(img->vdata,img->dim);
 	rdata=fits_image_first_pixel(ret->vdata,ret->dim);
+	/*
+	fprintf(stderr,"rdata=%p\n",(void *)rdata);
+	fprintf(stderr,"ret->vdata=%p\n",(void *)ret->vdata);
+	*/
         if ( flag )     memcpy(rdata,idata,sizeof(double)*ntot);
         else            memset(rdata,0,sizeof(double)*ntot);
   }
@@ -792,4 +799,5 @@ int fits_image_trim(fitsimage *img,int x0,int y0,int nx,int ny,double outer)
 	NASA Goddard Space Flight Center
 	http://fits.gsfc.nasa.gov/fits_documentation.html
 ******************************************************************************/
-                                                                       
+                                                           
+                         
