@@ -244,7 +244,8 @@ int aperture_photometry_back_int(double **data,char **mask,int sx,int sy,double
  return(0);
 }
 
-int get_histogram_peak(double *list,int n,double width,double *ret)
+/*
+static int get_histogram_peak(double *list,int n,double width,double *ret)
 {
  double	fmin,cv,sum;
  int	kmin,kmax,lmin,i;
@@ -278,6 +279,7 @@ int get_histogram_peak(double *list,int n,double width,double *ret)
 
  return(0); 
 }
+*/
 
 int aperture_photometry_back_ring(double **data,char **mask,int sx,int sy,
 	double cx,double cy,apphotpar *par,
@@ -406,7 +408,12 @@ int aperture_photometry_back_ring(double **data,char **mask,int sx,int sy,
  *rbgarea=(double)nbg;
  *rbgflux=flx;
  *rbgavg=avg;
- *rbgsigma=sqrt(std);
+ if ( par->bgm.scatter==BGSCATTER_STDDEV )
+	*rbgsigma=sqrt(std);
+ else if ( par->bgm.scatter==BGSCATTER_MAD )
+	*rbgsigma=sig*1.4826022185056;
+ else
+	*rbgsigma=sqrt(std);
 
  free(bds);
  free(bgs);
@@ -573,7 +580,12 @@ int aperture_photometry_back_polygons(double **data,char **mask,int sx,int sy,
  *rbgarea=(double)nbg;
  *rbgflux=flx;
  *rbgavg=avg;
- *rbgsigma=sqrt(std);
+ if ( par->bgm.scatter==BGSCATTER_STDDEV )
+	*rbgsigma=sqrt(std);
+ else if ( par->bgm.scatter==BGSCATTER_MAD )
+	*rbgsigma=sig*1.4826022185056;
+ else
+	*rbgsigma=sqrt(std);
 
  free(bds);
  free(bgs);
@@ -811,7 +823,13 @@ int aperture_photometry_flux_polygon(double **data,char **mask,int sx,int sy,
 		xpolypoints[2*k+1]=poly[2*k+1]+cy;
 	 }
 	nxpolypoint=polygon_intersection_square(xpolypoints,nppoly,(double)j,(double)i,1.0,1.0);
+	for ( k=0 ; k<nxpolypoint ; k++ )
+	 {	xpolypoints[2*k+0] -= cx;
+		xpolypoints[2*k+1] -= cy;
+	 }
+
 	a=polygon_area(xpolypoints,nxpolypoint);
+	polygon_integrate_linear_monoms(xpolypoints,nxpolypoint,fcoeff);
 
 	cflag=0;
 
