@@ -513,7 +513,7 @@ int overscan_model(int sy,double *spy,double *spw,double *spm,overscan *o,fitshe
  return(0);
 }
 
-int overscan_do_vertical(fitsimage *img,char **mask,int x0,int y0,int sx,int sy,int ox,int wx,overscan *o,compar *cp,fitsheaderset *header)
+int overscan_do_vertical(fitsimage *img,char **mask,int x0,int y0,int sx,int sy,int ox,int wx,overscan *o,combine_parameters *cp,fitsheaderset *header)
 {
  int	i,j,lp,np,x,y;
  double	*line,*spw,*spy,*spm;
@@ -571,7 +571,7 @@ int overscan_do_vertical(fitsimage *img,char **mask,int x0,int y0,int sx,int sy,
  return(0);
 }
 
-int overscan_do_horizontal(fitsimage *img,char **mask,int x0,int y0,int sx,int sy,int oy,int wy,overscan *o,compar *cp,fitsheaderset *header)
+int overscan_do_horizontal(fitsimage *img,char **mask,int x0,int y0,int sx,int sy,int oy,int wy,overscan *o,combine_parameters *cp,fitsheaderset *header)
 {
  int	i,j,lp,np,x,y;
  double	*line,*spw,*spy,*spm;
@@ -640,7 +640,7 @@ int overscan_check(trim *area,trim *o)
 	return(0);	/* invalid overscan	*/
 }
 
-int overscan_correction(fitsimage *img,char **mask,trim *area,overscan *scans,int nscan,compar *cp,fitsheaderset *header)
+int overscan_correction(fitsimage *img,char **mask,trim *area,overscan *scans,int nscan,combine_parameters *cp,fitsheaderset *header)
 {
  int		ret,r,i;
  overscan	*o;
@@ -674,7 +674,7 @@ int overscan_correction(fitsimage *img,char **mask,trim *area,overscan *scans,in
 
 /*
 fits *combine_more_images(char **files,int nfile,int frameno,
-	compar *cp,calibdata *cd,presubdata *presubs,int npresub)
+	combine_parameters *cp,calibdata *cd,presubdata *presubs,int npresub)
 {
  FILE	*fr;
  fits	*img,*ret;
@@ -829,7 +829,7 @@ double calibrate_get_flat_mean(fitsimage *flat,char **mask)
 }
 
 int calibrate_image(char *input,char *output,
-	geometry *g,mosaic *m,calibdata *cd,compar *cpover,fitsdataparam *fdp,headername *hn,
+	geometry *g,mosaic *m,calibdata *cd,combine_parameters *cpover,fitsdataparam *fdp,headername *hn,
 	fitsimage *bias,fitsimage *dark,fitsimage *flat,
 	double flatmean,double darktime,int no_history)
 {
@@ -1175,7 +1175,7 @@ int calibrate_image(char *input,char *output,
 
  if ( had_flat )
   {	fitsheaderset	*header;
-	char		buff[80],bcmm[80];
+	char		buff[80],bcmm[512];
 	header=&img->header;
 	snprintf(buff,80,"ficalib: flat: mean=%.1f flat=\"%s\"",flatmean,cd->inputflat);
 	fits_headerset_set_string(header,"FLATCOR",FITS_SH_ADD,buff,NULL);
@@ -1667,20 +1667,20 @@ int ficalib_parse_mosaic_data(char *mstr,mosaic *m)
 
 int main(int argc,char *argv[])
 {
- char		**inputlist,**outputlist,**inmasklist,*rewriterule;
- fits		*bias,*dark,*flat;
- double		flatmean,darktime;
- FILE		*fr;
- int		ninimg;
- int		i,n,is_help,apply_mask,no_rewrite,no_history;
- char		*fdpstring,*overmodestr,*imgarea,*gainstr,
-		*hdrdefstr,**overscanlist,*mosaicstr;
- compar		cpover;
- fitsdataparam	fdp;
- calibdata	cd;
- headername	hn;
- geometry	g;
- mosaic		m;
+ char			**inputlist,**outputlist,**inmasklist,*rewriterule;
+ fits			*bias,*dark,*flat;
+ double			flatmean,darktime;
+ FILE			*fr;
+ int			ninimg;
+ int			i,n,is_help,apply_mask,no_rewrite,no_history;
+ char			*fdpstring,*overmodestr,*imgarea,*gainstr,
+			*hdrdefstr,**overscanlist,*mosaicstr;
+ combine_parameters	cpover;
+ fitsdataparam		fdp;
+ calibdata		cd;
+ headername		hn;
+ geometry		g;
+ mosaic			m;
 
  progbasename=strrchr(argv[0],'/');
  if ( progbasename != NULL )	progbasename++;
@@ -1696,7 +1696,8 @@ int main(int argc,char *argv[])
  fdp.bitpix=-32;fdp.is_scale=0;fdp.bscale=1.0;fdp.bzero=0.0;
  fdpstring=NULL,hdrdefstr=NULL,overscanlist=NULL;
 
- cpover.mode=COM_MODE_MED;
+ combine_parameters_reset(&cpover);
+ cpover.mode=COM_MODE_MEDIAN;
  cpover.ignore_flag=0;
  cpover.logicalmethod=0;
  cpover.niter=0;
