@@ -346,7 +346,8 @@ int make_subtracted_image(fitsimage *cnv,char **mask,fitsimage *img,char **mask_
  if ( img==NULL || img->data==NULL )	return(1);
  if ( out==NULL || out->data==NULL )	return(1);
  sx=cnv->sx,sy=cnv->sy;
- 
+
+ /* this xlist != NULL part is needed to be obsoleted sooner-or-later: */ 
  if ( xlist != NULL && xlist->kernels != 0 && xlist->nkernel>0 )
   {	kernel	*k;
 	int	ihsize;
@@ -634,14 +635,14 @@ int fprint_ficonv_usage(FILE *fw)
  fprintf(fw,
 "Usage:\tficonv [-h|--help|--long-help|--wiki-help] [--version[-short]]\n"
 "\t[-C|--comment] [-V|--verbose]\n"
-"\t[-r <reference.fits>] [-i <input-to-fit.fits>\n"
+"\t[-r|--reference|--input-reference <reference.fits>]\n"
+"\t[-i|--input <input-to-fit.fits>\n"
 "\t[-m|--masked [-w|--weighted]] [-b|--background-iterative]\n"
 "\t[-n|--iterations <niter>] [-s|--rejection-level <rejlevel>]]\n"
-"\t{--input-kernel <input-kernels>|-k \"<input-kernel-set>\"}\n"
-"\t[--output-kernel <output-kernels>]\n");
+"\t{ --input-kernel-list <input-kernel-list-file> |\n"
+"\t  -k|--kernel|--kernels \"<input-kernel-set>\" }\n"
+"\t[--output-kernel-list <output-kernel-list-file> [-u|--unity-kernels]]\n");
  fprintf(fw,
-"\t[--input-extra-kernel <pdcnv-kernels>|-x \"<pdcnv-kernel-set>\"]"
-"\t[--output-extra-kernel <output-pdcnv>]\n"
 "\t[-o|--output <output-convolved.fits>]\n"
 "\t[--output-subtracted <output-subtracted.fits>]\n"
 "\t[-a <add-to-convolved.fits>] [-d|--divide <blocks>]\n"
@@ -663,23 +664,23 @@ longhelp_entry ficonv_long_help[]=
 	"Gives a detailed list of command line options in Mediawiki format." },
  { "--version, --version-short, --short-version",
 	"Gives some version information about the program." },
- { "-r, --input-reference <image file>",
+ { "-r, --reference, --input-reference <image file>",
 	"Name of the reference FITS image file." },
  { "-i, --input <image file>",
 	"Name of the input FITS image file (required only for kernel fitting)." },
- { "-k, --kernel <kernel set>",
+ { "-k, --kernel, --kernels <kernel set>",
 	"List of kernel bases used for fitting convolution kernel." 
 	"See also \"Kernel specifications\" below for the format of this "
 	"<kernel set> argument." },
- { "--input-kernel <file>",
+ { "--input-kernel-list <file>",
 	"Name of the file containing kernel bases. The kernel bases in this "
 	"file should have no associated coefficients if convolution fitting "
 	"is done, otherwise the kernel basis file must contain the "
 	"convolution coefficients" },
- { "--output-kernel <file>",
+ { "--output-kernel-list <file>",
 	"Name of the file where the coefficients for the kernel bases are "
 	"stored after convolution kernel fitting" },
- { "-o, -oc, --output, --output-convolved <image file>", 
+ { "-o, --output, --output-convolved <image file>", 
 	"Name of the output file which is the reference image convolved with "
 	"the kernel solution (which can either be a previously fitted and now "
 	"read from a file or the result of the current fit)" },
@@ -746,9 +747,10 @@ int main(int argc,char *argv[])
  double		gain,rejectlevel;
  int		psx,psy;
 
- char		*imgname,*refname,*ikname,*okname,*ixname,*oxname,
+ char		*imgname,*refname,*ikname,*okname,
  		*oconvname,*osubname,*addname,**inmasklist,*kernelarg,
-		*stamparg,*stampfile,*splinestampfile,*xkarg;
+		*stamparg,*stampfile,*splinestampfile;
+ int		unity_kernels;
  kernellist	klist_data,*klist=&klist_data,
 		xlist_data,*xlist=&xlist_data;
 
@@ -760,12 +762,13 @@ int main(int argc,char *argv[])
  else				progbasename=argv[0];
 
  is_comment=is_verbose=is_help=0;
- addname=imgname=refname=ikname=okname=ixname=oxname=oconvname=osubname=NULL;
- stamparg=kernelarg=xkarg=NULL; inmasklist=NULL;
+ addname=imgname=refname=ikname=okname=oconvname=osubname=NULL;
+ stamparg=kernelarg=NULL; inmasklist=NULL;
  stampfile=splinestampfile=NULL;
  method=0; gain=0.0; niter=0;rejectlevel=3.0;
  bdc=32;
  psx=psy=-1;
+ unity_kernels=0;
 
  i=scanarg(argc,argv,SCANARG_ALLOW_FLAGS,
 	"--version:%NS-1f%q",&is_help,
@@ -778,7 +781,7 @@ int main(int argc,char *argv[])
 	"--output-subtracted:%s",&osubname,
 	"-i|--input|-f|--fit:%s",&imgname,
 	"-M|--input-mask:%t",&inmasklist,
-	"-r|--reference:%s",&refname,
+	"-r|--input-reference|--reference:%s",&refname,
 	"-a|--add-to:%s",&addname,
 	"-t|--stamp|--stamps:%s",&stamparg,
 	"--input-stamp|--input-stamps:%s",&stampfile,
@@ -786,10 +789,8 @@ int main(int argc,char *argv[])
 	"--input-spline-stamp|--input-spline-stamps:%s",&splinestampfile,
 	"--input-kernel-list:%s",&ikname,
 	"--output-kernel-list:%s",&okname,
-	"-k|--kernel:%s",&kernelarg,
-	"--input-extra-kernel-list:%s",&ixname,
-	"--output-extra-kernel-list:%s",&oxname,
-	"-x|--extra-kernel:%s",&xkarg,
+	"-u|--unity-kernels:%f",&unity_kernels,
+	"-k|--kernel|--kernels:%s",&kernelarg,
 	"-m|--masked:%0f",&method,
 	"-w|--weighted:%1f",&method,
 	"-b|--background-iterative:%2f",&method,
@@ -937,16 +938,10 @@ int main(int argc,char *argv[])
   {	fprint_error("ambiguous kernel definitions");
 	return(1);
   }
- if ( xkarg != NULL && ixname != NULL )
-  {	fprint_error("ambiguous reference kernel definitions");
-	return(1);
-  }
  if ( kernelarg == NULL && ikname == NULL )
   {	fprint_error("convolution information is not known/specified");
 	return(1);
   }
- if ( xkarg == NULL && ixname == NULL )
-	oxname=NULL;	/* ignore -ox if -ix or -x is not specified */
  if ( addname != NULL && imgname != NULL )
   {	fprint_error("invalid combination of input image specifications");
 	return(1);
@@ -983,34 +978,8 @@ int main(int argc,char *argv[])
 
  xlist->nkernel=0;
  xlist->kernels=NULL;
- if ( xkarg != NULL )
-  {	int	r;
-	r=create_kernels_from_kernelarg(xkarg,xlist);
-	xlist->type=0;
-	if ( r )
-	 {	fprint_error("unable to parse reference kernel specification list");
-		return(1);
-	 }
-  }
- if ( ixname != NULL )
-  {	FILE	*fr;
-	int	r;
-	fr=fopenread(ixname);
-	if ( fr==NULL )
-	 {	fprint_error("unable to open reference kernel list file '%s'",ixname);
-		return(1);
-	 }
-	r=kernel_info_read(fr,xlist);
-	fcloseread(fr);
-	if ( r )
-	 {	fprint_error("unable to read or parse contents of reference kernel list file '%s'",ixname);
-		return(1);
-	 }
-  }
- for ( i=0 ; xlist->kernels != NULL && i<xlist->nkernel ; i++ )
-  {	xlist->kernels[i].target=1;		}
 
- logmsg(is_verbose>=1,"Number of kernels: %d+%d.\n",klist->nkernel,xlist->nkernel);
+ logmsg(is_verbose>=1,"Number of kernels: %d.\n",klist->nkernel);
  
  kernel_init_images(klist);
  kernel_init_images(xlist);
@@ -1126,7 +1095,7 @@ int main(int argc,char *argv[])
 		 }
 	 }
 
-        fits_history_export_command_line(img,"ficonv",FITSH_FICONV_VERSION,argc,argv);
+        fits_history_export_command_line(outimg,"ficonv",FITSH_FICONV_VERSION,argc,argv);
 
 	if ( oconvname != NULL )
 	 {	FILE	*fw;
@@ -1167,19 +1136,34 @@ int main(int argc,char *argv[])
 	 {	fprint_error("unable to create output kernel list file '%s'",okname);
 		return(1);
 	 }
+	if ( unity_kernels )
+	 {	int	i;
+		double	norm;
+		norm=1.0;
+		for ( i=0 ; i<klist->nkernel ; i++ )
+		 {	kernel	*k;
+			k=&klist->kernels[i];
+			if ( k->type==KERNEL_IDENTITY )
+			 {	int	o;
+				norm=k->coeff[0];
+				k->coeff[0]=1.0;
+				for ( o=1 ; o<(k->order+1)*(k->order+2)/2 ; o++ )
+					k->coeff[o]=0.0;
+			 }
+		 }
+		for ( i=0 ; i<klist->nkernel ; i++ )
+		 {	kernel	*k;
+			k=&klist->kernels[i];
+			if ( k->type==KERNEL_DDELTA && 0<norm )
+			 {	int	o;
+				for ( o=0 ; o<(k->order+1)*(k->order+2)/2 ; o++ )
+					k->coeff[o] /= norm;
+			 }
+		 }
+	 }	
 	kernel_info_write(fw,klist);
 	fclosewrite(fw);
   }
- if ( oxname != NULL )
-  {	FILE	*fw;
-	fw=fopenwrite(oxname);
-	if ( fw==NULL )	
-	 {	fprint_error("unable to create output reference kernel list file '%s'",oxname);
-		return(1);
-	 }
-	kernel_info_write(fw,xlist);
-	fclosewrite(fw);
-  }
 
  if ( mask_ref != NULL )	fits_mask_free(mask_ref);
  if ( mask_img != NULL )	fits_mask_free(mask_img);
