@@ -3,9 +3,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Command line tool for exporting FITS headers to file/standard output.     */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* (c) 2004, 2006; Pal, A. (apal@szofi.elte.hu)				     */
+/* (c) 2004, 2006, 2017; Pal, A. <apal@szofi.net>			     */
 /*****************************************************************************/
-#define	FITSH_FIHEADER_VERSION	"0.9c1"
+#define	FITSH_FIHEADER_VERSION	"0.9d"
 /*****************************************************************************/
 
 #include <stdio.h>
@@ -26,6 +26,7 @@
 #include "io/scanarg.h"
 #include "io/iof.h"
 #include "io/tokenize.h"
+#include "str.h"
 
 /*****************************************************************************/
 
@@ -33,6 +34,8 @@
 #define		FORMAT_KEYWORD		2
 #define		FORMAT_VALUE		3
 #define		FORMAT_COMMENT		4
+#define		FORMAT_LIST		5
+#define		FORMAT_EQUAL_SIGN	15
 
 #define		ALTER_UPDATE		1
 #define		ALTER_APPEND		2
@@ -41,10 +44,10 @@
 /*****************************************************************************/
 
 typedef struct
- {	char	keyword[16];
+ {	char	keyword[80];
 	int	occurrence;
-	char	value[72];
-	char	comment[72];
+	char	value[80];
+	char	comment[80];
  } alter;
 
 /*****************************************************************************/
@@ -103,13 +106,15 @@ int fprint_header_value(FILE *fw,fitsheader *hdr)
 
 /*****************************************************************************/
 
-int fprint_formatted_keyword(FILE *fw,char *sfbuff,char *name,char *keyw,int *format,fitsheader *hdr)
+int fprint_formatted_keyword(FILE *fw,char *sfbuff,char *name,char *keyw,int *format,fitsheader *hdr,int is_newline)
 {
  int	f;
 
  for ( f=0 ; format[f]>=0 ; f++ )
   {	if ( f>0 )
-	 {	fprintf(fw," ");	}	
+		fprintf(fw," ");
+	else if ( ! is_newline )
+		fprintf(fw,"\t");
 
 	switch ( format[f] )
 	 {   case FORMAT_FILE:
@@ -128,9 +133,14 @@ int fprint_formatted_keyword(FILE *fw,char *sfbuff,char *name,char *keyw,int *fo
 		if ( hdr != NULL )
 		 {	fprintf(fw,"# %s",hdr->comment);	}
 		break;
+	     case FORMAT_EQUAL_SIGN:
+		fprintf(fw,"=");
+		break;
   	 }
   }
- fprintf(fw,"\n");
+ if ( is_newline )
+	fprintf(fw,"\n");
+
  return(0);
 }
 
@@ -145,7 +155,7 @@ int is_any_wildcard(char *buff)
  return(0);
 }
 
-int fprint_keyword(FILE *fw,fitsheaderset *header,char *name,char *keyw,int *format,int namelen)
+int fprint_keyword(FILE *fw,fitsheaderset *header,char *name,char *keyw,int *format,int namelen,int is_newline)
 {
  char		sfbuff[16];
  int		n,j;
@@ -156,10 +166,10 @@ int fprint_keyword(FILE *fw,fitsheaderset *header,char *name,char *keyw,int *for
  if ( ! is_any_wildcard(keyw) )
   {	n=fits_headerset_get_count(header,keyw);
 	if ( n==0 )
-	 {	fprint_formatted_keyword(fw,sfbuff,name,keyw,format,NULL);	}
+	 	fprint_formatted_keyword(fw,sfbuff,name,keyw,format,NULL,is_newline);
 	for ( j=0 ; j<n ; j++ )
 	 {	hdr=fits_headerset_get_header(header,keyw,j);
-		fprint_formatted_keyword(fw,sfbuff,name,keyw,format,hdr);
+		fprint_formatted_keyword(fw,sfbuff,name,keyw,format,hdr,is_newline);
 	 }
   }
  else
@@ -171,7 +181,7 @@ int fprint_keyword(FILE *fw,fitsheaderset *header,char *name,char *keyw,int *for
 #else
 		if ( strcmp(keyw,hdr->name)==0 )
 #endif
-		 {	fprint_formatted_keyword(fw,sfbuff,name,hdr->name,format,hdr);
+		 {	fprint_formatted_keyword(fw,sfbuff,name,hdr->name,format,hdr,is_newline);
 			n++;
 		 }
 		
@@ -182,14 +192,14 @@ int fprint_keyword(FILE *fw,fitsheaderset *header,char *name,char *keyw,int *for
 
 }
 
-int read_header_values_from_file(FILE *fr,char *extension,char *name,char **getkwlist,FILE *fw,int *format,int namelen)
+int read_header_values_from_file(FILE *fr,char *extension,char *name,char **keywordlist,FILE *fw,int *format,int namelen)
 { 
  fits		*img;
  fitsheaderset	*header;
- int		i,k,nkw,ret;
- char		**cmd,*kwl;
+ int		i,ret,is_list;
 
- if ( getkwlist==NULL )		return(0);
+ if ( keywordlist==NULL || format==NULL )
+	return(0);
 
  if ( extension==NULL )
   {	img=fits_create();
@@ -222,18 +232,35 @@ int read_header_values_from_file(FILE *fr,char *extension,char *name,char **getk
 
  ret=0;
 
- for ( i=0 ; getkwlist[i] != NULL ; i++ )
-  {	kwl=strdup(getkwlist[i]);
-	remove_spaces(kwl);
-	for ( k=0,nkw=1 ; kwl[k] ; k++ )
-	 {	if ( kwl[k]==',' )	nkw++;
-		else			kwl[k]=toupper((int)kwl[k]);
+ for ( i=0,is_list=0 ; 0<=format[i] ; i++ )
+  {	if ( format[i]==FORMAT_LIST )
+		is_list=1;
+  }
+
+ if ( is_list )
+  {	int	i,fformat[2],vformat[2];
+	fformat[0]=FORMAT_FILE;
+	vformat[0]=FORMAT_VALUE;
+	fformat[1]=vformat[1]=-1;
+	for ( i=0 ; 0<=format[i] ; i++ )
+	 {	if ( format[i]==FORMAT_FILE )
+		 {	if ( 0<i )	fprintf(fw,"\t");
+			fprint_keyword(fw,header,name,"",fformat,namelen,0);
+		 }
+		else if ( format[i]==FORMAT_LIST )
+		 {	int	k;
+			for ( k=0 ; keywordlist[k] != NULL ; k++ )
+			 {	if ( fprint_keyword(fw,header,name,keywordlist[k],vformat,namelen,0) )
+					ret=1;
+			 }			
+		 }
 	 }
-	cmd=(char **)malloc(sizeof(char *)*(nkw+1));
-	tokenize_char(kwl,cmd,',',nkw+1);
-	
-	for ( k=0 ; k<nkw ; k++ )
-	 {	if ( fprint_keyword(fw,header,name,cmd[k],format,namelen) )
+	fprintf(fw,"\n");
+  }
+ else
+  {	int	k;
+	for ( k=0 ; keywordlist[k] != NULL ; k++ )
+	 {	if ( fprint_keyword(fw,header,name,keywordlist[k],format,namelen,1) )
 			ret=1;
 	 }
   }
@@ -279,7 +306,7 @@ int alter_get_action(char *act)
 
 int alter_is_keyword_char(int t)
 {
- if ( t && t != '=' && t != ',' && (!isspace(t)) && t != '[' && t != ']' )
+ if ( t && t != '=' && t != ',' && t != '[' && t != ']' )
 	return(1);
  else	
 	return(0);
@@ -300,7 +327,7 @@ int alter_get_alternations(char *list,alter **ralters,int *rnalter,int is_set)
  while ( *p )
   {	while ( isspace((int)*p) )	p++;
 	lk=0;
-	while ( alter_is_keyword_char(*p) && lk<8 )
+	while ( alter_is_keyword_char(*p) && lk<80 )
 	 {	sk[lk]=toupper((int)*p),lk++,p++;		}
 	sk[lk]=0;
 	while ( isspace((int)*p) )		p++;
@@ -512,7 +539,7 @@ int fprint_fiheader_usage(FILE *fw)
  fprintf(fw,
 "Read header/keyword values:\n"
 "\t[--get|--read <KEYWORD1>[,<KEYWORD2>,...]] [--get|--read ...]\n"
-"\t[-F|--format file[name],keyword,value,comment]\n");
+"\t[-F|--format file[name],{keyword,value,comment|list}]\n");
  fprintf(fw,
 "Altering keyword values:\n"
 "\t[--set|--update <KEYWORD1>=<value>[/comment][,<KEYWORD2>=...,...]]\n"
@@ -560,8 +587,9 @@ longhelp_entry fiheader_long_help[]=
 	"List of keywords to get from FITS header." },
  { "-F, --format <format>", 
 	"Format of the output, space separated list of the arguments "
-	"\"filename\", \"keyword\", \"value\" or \"comment\", in the "
-	"desired order." },
+	"\"filename\", \"keyword\", \"value\", \"comment\" or \"list\", in "
+	"the desired order. The \"list\" here is a list of tab-separated "
+	"values of matched keywords." },
  { "-x, --extension <extension>",
 	"Read keyword values from the given extension." },
  { "Note that shell wildcard character patterns can be used in the "
@@ -600,6 +628,13 @@ int fprint_fiheader_long_help(FILE *fw,int is_wiki)
  return(0);
 }
 
+static int string_toupper(char *str)
+{
+ for ( ; *str; str++) 
+  {	*str=toupper(*str);	}
+ return(0);
+}
+
 /*****************************************************************************/
 
 static	int	format_v  []= { FORMAT_VALUE,-1};
@@ -674,21 +709,26 @@ int main(int argc,char *argv[])
 	format=NULL;
  else
   {	char	*af,*cmd[8];
-	int	i,n;
+	int	i,n,is_list,is_other;
 	format=(int *)malloc(8*sizeof(int));
 	af=strdup(argformat);
 	remove_spaces(af);
 	n=tokenize_char(af,cmd,',',7);
+	is_list=is_other=0;
 	for ( i=0 ; cmd[i] != NULL && i<n ; i++ )
 	 {	if ( strcmp(cmd[i],"file")==0 || strcmp(cmd[i],"filename")==0 ||
 		strcmp(cmd[i],"name")==0 )
 		 	format[i]=FORMAT_FILE;
 		else if ( strcmp(cmd[i],"keyword")==0 )
-			format[i]=FORMAT_KEYWORD;
+			format[i]=FORMAT_KEYWORD,is_other=1;
 		else if ( strcmp(cmd[i],"value")==0 )
-			format[i]=FORMAT_VALUE;
+			format[i]=FORMAT_VALUE,is_other=1;
 		else if ( strcmp(cmd[i],"comment")==0 )
-			format[i]=FORMAT_COMMENT;
+			format[i]=FORMAT_COMMENT,is_other=1;
+		else if ( strcmp(cmd[i],"list")==0 )
+			format[i]=FORMAT_LIST,is_list=1;
+		else if ( strcmp(cmd[i],"=")==0 )
+			format[i]=FORMAT_EQUAL_SIGN,is_other=1;
 		else
 		 {	fprint_error("invalid format specification '%s'",argformat);
 			return(1);
@@ -696,6 +736,12 @@ int main(int argc,char *argv[])
 	 }
 	format[i]=-1;
 	free(af);
+	
+	if ( is_other && is_list )
+	 {	fprint_error("invalid combination of format parameters: \"list\" cannot be used in conjunction with \"keyword\", \"value\" or \"comment\"");
+		return(1);
+	 }
+
   }
 	
  
@@ -718,10 +764,18 @@ int main(int argc,char *argv[])
   }
 
  if ( (method & 1) && getkwlist != NULL )	/* read some headers */
-  {	gkwcnt=0;
+  {	char	*keywordstring=NULL,**keywordlist;
+
+	/* concatenate+split keyword list (passed via a -g or multiple -g): */
 	for ( i=0 ; getkwlist[i] != NULL ; i++ )
-	 {	for ( j=0 ; getkwlist[i][j] ; j++ )
-		 {	if ( getkwlist[i][j]==',' )	gkwcnt++;	}
+	 {	if ( keywordstring==NULL )
+			keywordstring=strdup(getkwlist[i]);
+		else
+			strappendf(&keywordstring,",%s",getkwlist[i]);
+	 }
+	keywordlist=tokenize_char_dyn(keywordstring,',');
+	for ( gkwcnt=0; keywordlist[gkwcnt] != NULL ; )	
+	 {	string_toupper(keywordlist[gkwcnt]);	
 		gkwcnt++;
 	 }
 
@@ -738,7 +792,7 @@ int main(int argc,char *argv[])
 
 	if ( incnt==0 )
 	 {	fr=stdin;
-		ret=read_header_values_from_file(fr,extension,"-",getkwlist,stdout,format,1);
+		ret=read_header_values_from_file(fr,extension,"-",keywordlist,stdout,format,1);
 	 }
 	else	
 	 {	ret=0;
@@ -748,10 +802,14 @@ int main(int argc,char *argv[])
 			 {	fprint_error("unable to open input file '%s'",innames[i]);
 				return(1);
 			 }
-			ret|=read_header_values_from_file(fr,extension,innames[i],getkwlist,stdout,format,1);
+			ret|=read_header_values_from_file(fr,extension,innames[i],keywordlist,stdout,format,1);
 			fcloseread(fr);
 		 }
 	 }
+
+	free(keywordlist);
+	free(keywordstring);
+
 	if ( ret )	return(2);	/* exit with errorlevel 2 */
   }
  else if ( (method & 2) && setkwlist != NULL )	/* alter some headers */
