@@ -233,11 +233,14 @@ int fprint_fiign_usage(FILE *fw)
 "\t[--ignore-mask | -M|--input-mask <mask>]\n"
 "\t[-a|--apply-mask [-m|--mask-value <value>]] [--output-mask <mask>]\n");
  fprintf(fw,
-"Mask conversion:\n"
-"\t[--convert <match>:<value>:<reset>:<set> [--convert <>:<>:<>:<>] ...]\n");
+"Mask conversion and expansion:\n"
+"\t[--convert <match>:<value>:<reset>:<set> [--convert <>:<>:<>:<>] ...]\n"
+"\t[-x|--expand <size>]\n"
+"\t[-q|--mask-block <shape>:<mask>:<coordinates_and_parameters>]\n");
+
  fprintf(fw,
 "Marking saturated pixels:\n"
-"\t[-s <saturation-level>|-S <saturation_image.fits>]\n"
+"\t[-s <saturation_level>|-S <saturation_image.fits>]\n"
 "\t[--leak-{left-right|lower-upper|any}|--lr|--lu|--an]\n");
  fprintf(fw,
 "General ignorance:\n"
@@ -294,13 +297,36 @@ longhelp_entry fiign_long_help[]=
  { "--leak-left-right, --leak-lower-upper, --leak-any",
 	"Readout direction, i.e. orientation of \"blooming\" stripes." },
 
- { "Mask conversion:", NULL },
+ { "Mask conversion and expansion:", NULL },
  { "--convert <match>:<value>:<reset>:<set>",
 	"Convert masks: from a mask which matches to the <match>:<value> pair, "
 	"i.e. the masks with the type of <match> have a value of <value> "
 	"the masks specified by <reset> are cleared and the masks specified by "
 	"<set> are set. The <match>, <value>, <reset> and <set> tags are "
 	"comma-separated list of mask names (see below)." },
+ { "-x, --expand <size>", 
+	"Expands the mask with the specified size. Applied after all of "
+	"the previous steps were performed. " }, 
+
+ { "Additional mask layers:", NULL },
+ { "-q, --mask-block block:<mask>:<x1>,<y1>:<x2>,<y2>",
+	"Add a rectangular masked area with the "
+	"specified mask <mask> and with the specified coordinates "
+	"(<x1>,<y1>: lower-left corner, <x2>,<y2>: upper-right corner, inclusive). " 
+ },
+ { "-q, --mask-block circle:<mask>:<xc>,<yc>:<radius>",
+	"Add a circular masked area with the "
+	"specified mask <mask> and with the specified cenroid coordinates <xc>,<yc> "
+	"and radius <radius>. "
+ },
+ { "-q, --mask-block pixel:<mask>:<x>,<y>",
+	"Add a masked pixel with the "
+	"specified mask <mask> to the specified coordinates <x>,<y>."
+ },
+ { "-q, --mask-block line:<mask>:<x1>,<y1>:<x2>,<y2>",
+	"Add a masked line with the "
+	"specified mask <mask> connecting the points <x1>,<y1> and <x2>,<y2>."
+ },
 
  { "Mask names:", NULL },
  { "none",
@@ -369,7 +395,7 @@ int main(int argc,char *argv[])
  char 		*satimgname,*inimgname,*outimgname,*outmaskname,**inmasklist,
 		*ignmasknames,**convertlist,*basename,**maskblocklist;
  int		ign_cosmics,rep_cosmics,ign_num,frameno,
-		reset_method,apply_mask;
+		reset_method,apply_mask,expand_mask_hsize;
  maskconvert	*mcls;
  int		nmcl;
 
@@ -385,6 +411,8 @@ int main(int argc,char *argv[])
  ignmasknames=NULL;inmasklist=convertlist=NULL;maskvalue=0.0;
  maskblocklist=NULL;
  mcls=NULL;nmcl=0;
+ 
+ expand_mask_hsize=0;
 
  i=scanarg(argc,argv,SCANARG_ALLOW_FLAGS,
 	"--version:%NS-1f%q",&is_help,
@@ -407,6 +435,7 @@ int main(int argc,char *argv[])
 	"-z|--ignore-zero:%N1f",&ign_num,
 	"-g|--ignore-negative:%N2f",&ign_num,
 	"-n|--ignore-nonpositive:%N3f",&ign_num,
+	"-x|--expand:%d",&expand_mask_hsize,
 	"-a|--apply-mask:%f",&apply_mask,
 	"-m|--mask-value:%g",&maskvalue,
 	"--no-mask|--ignore-mask:%f",&is_no_mask,
@@ -594,7 +623,9 @@ int main(int argc,char *argv[])
 		 }
 	 }
   }
-			
+
+ if ( 0<expand_mask_hsize )
+	fits_mask_expand_logic(mask,sx,sy,expand_mask_hsize,-1,0,0);
 
  if ( apply_mask )
   {	for ( i=0 ; i<sy ; i++ )
