@@ -23,6 +23,7 @@
 #include "magnitude.h"
 #include "math/poly.h"
 
+#include "str.h"
 #include "common.h"
 #include "kernel.h"
 
@@ -30,6 +31,136 @@
 
 /*****************************************************************************/
 
+static int aperture_definition_create_polygon_regular(double **polypoints,int *npolypoint,double requiv,int nvertex,double alpha)
+{
+ double	*poly,r;
+ int	i;
+
+ r=requiv*sqrt((2*M_PI/(double)nvertex)/sin(2*M_PI/(double)nvertex));
+ alpha=alpha*M_PI/180.0;
+
+ poly=(double *)malloc(sizeof(double)*2*nvertex);
+ for ( i=0 ; i<nvertex ; i++ )
+  {	poly[2*i+0]=-r*sin(2*M_PI/(double)nvertex*((double)i+0.5)+alpha);
+	poly[2*i+1]=+r*cos(2*M_PI/(double)nvertex*((double)i+0.5)+alpha);
+	/* fprintf(stderr,"%12g %12g\n",poly[2*i+0],poly[2*i+1]); */
+  }
+
+ *polypoints=poly;
+ *npolypoint=nvertex;
+
+ return(0);
+}
+
+static int aperture_definition_create_polygon_trail(double **polypoints,int *npolypoint,double requiv,int nvertex,double dx,double dy)
+{
+ double	*poly,r,alpha;
+ int	i;
+
+ r=requiv*sqrt((2*M_PI/(double)nvertex)/sin(2*M_PI/(double)nvertex));
+ alpha=atan2(dy,dx);
+
+ poly=(double *)malloc(sizeof(double)*2*nvertex);
+ for ( i=0 ; i<nvertex ; i++ )
+  {	poly[2*i+0]=-r*sin(2*M_PI/(double)nvertex*((double)i+0.5)+alpha)+(i<nvertex/2?-dx/2.0:dx/2.0);
+	poly[2*i+1]=+r*cos(2*M_PI/(double)nvertex*((double)i+0.5)+alpha)+(i<nvertex/2?-dy/2.0:dy/2.0);
+	/* fprintf(stderr,"%12g %12g\n",poly[2*i+0],poly[2*i+1]); */
+  }
+
+ *polypoints=poly;
+ *npolypoint=nvertex;
+
+ return(0);
+}
+
+static int aperture_definition_create_polygon(double **polypoints,int *npolypoint,char *def)
+{
+ int	i,n;
+ char	**cmd;
+ double	*arr;
+ while ( *def && *def != '[' )	def++;
+ if ( *def=='[' )		def++;
+ cmd=tokenize_char_dyn(def,',');
+ if ( cmd==NULL )	
+	return(1);
+ for ( n=0 ; cmd[n] != NULL ; )	n++;
+ if ( n<=0 || 0<(n%2) )
+  {	free(cmd);
+	return(1);
+  }
+ *npolypoint=n/2;
+ *polypoints=arr=(double *)malloc(sizeof(double)*n);
+ for ( i=0 ; i<n ; i++ )
+  {	if ( sscanf(cmd[i],"%lg",&arr[i])<1 )
+	 {	free(arr);
+		free(cmd);
+		return(1);
+	 }
+  }
+ free(cmd);
+ return(0);
+}
+
+static int aperture_definition_parse(char *apdef,apgeom *ap)
+{
+ double	r0,ra,da;
+
+ if ( sscanf(apdef,"%lg:%lg:%lg",&r0,&ra,&da)==3 && ( 0.0<r0 && 0.0<=ra && 0.0<da ) )
+  {	ap->apgeom_type=APGEOM_TYPE_CIRCULAR;
+	ap->r0=r0;
+	ap->ra=ra;
+	ap->da=da;
+	return(0);
+  }
+ else	
+  {	int	i,n;
+	char	*cmd[8];
+
+	n=tokenize_char(apdef,cmd,':',63);
+	if ( n != 3 )
+		return(1);
+
+	ap->apgeom_type=APGEOM_TYPE_POLYGON;
+
+	ap->r0_poly=NULL;
+	ap->ra_poly=NULL;
+	ap->da_poly=NULL;
+	ap->nr0=0;
+	ap->nra=0;
+	ap->nda=0;
+
+	for ( i=0 ; i<3 ; i++ )
+	 {	double	*polypoints;
+		int	npolypoint;
+
+		double	requiv,alpha,dx,dy;
+		int	nvertex,r;
+
+		if ( sscanf(cmd[i],"Q[%lg,%d,%lg]",&requiv,&nvertex,&alpha)==3 || sscanf(cmd[i],"regular[%lg,%d,%lg]",&requiv,&nvertex,&alpha)==3 )
+			r=aperture_definition_create_polygon_regular(&polypoints,&npolypoint,requiv,nvertex,alpha);
+		else if ( ( sscanf(cmd[i],"T[%lg,%d,%lg,%lg]",&requiv,&nvertex,&dx,&dy)==4 || sscanf(cmd[i],"trail[%lg,%d,%lg,%lg]",&requiv,&nvertex,&dx,&dy)==4 ) && (nvertex%2)==0 )
+			r=aperture_definition_create_polygon_trail(&polypoints,&npolypoint,requiv,nvertex,dx,dy);
+		else if ( ( memcmp(cmd[i],"P[",2)==0 || memcmp(cmd[i],"polygon[",8)==0 ) && cmd[i][strlen(cmd[i])-1]==']' )
+			r=aperture_definition_create_polygon(&polypoints,&npolypoint,cmd[i]);
+	 	else
+			return(1);
+
+		if ( r )
+			return(1);
+
+		if ( i==0 )		ap->r0_poly=polypoints,ap->nr0=npolypoint;
+		else if ( i==1 )	ap->ra_poly=polypoints,ap->nra=npolypoint;
+		else			ap->da_poly=polypoints,ap->nda=npolypoint;
+	 }
+
+	return(0);
+
+  }
+
+}
+
+/*****************************************************************************/
+
 int read_input_star_list(FILE *fr,photstar **rps,int *rnp,colread *col,int zoom)
 {
  photstar	*ps;
@@ -195,10 +326,13 @@ int read_raw_photometry(FILE *fr,photstar **rps,int *rnp)
 
 	inaps=(apgeom *)malloc(sizeof(apgeom)*n);
 	for ( i=0 ; i<n ; i++ )
-	 {	inaps[i].apgeom_type=APGEOM_TYPE_CIRCULAR;
-		sscanf(cmd[6+i*6+0],"%lg",&inaps[i].r0);
-		sscanf(cmd[6+i*6+1],"%lg",&inaps[i].ra);
-		sscanf(cmd[6+i*6+2],"%lg",&inaps[i].da);
+	 {	apgeom	ap;
+		char	*apdef;
+		apdef=NULL;
+		strappendf(&apdef,"%s:%s:%s",cmd[6+i*6+0],cmd[6+i*6+1],cmd[6+i*6+2]);
+		aperture_definition_parse(apdef,&ap);
+		free(apdef);
+		inaps[i]=ap;
 	 }
 	ps[np].inaps=inaps;
 
@@ -260,7 +394,7 @@ int get_id_format_parameters(photstar *ps,int np,int *rid_len,char *idf,char *nd
 int write_output_star_list(FILE *fw,photstar *ps,int np)
 {
  int	i,id_len;
- char	id_format[8],nd_format[8];
+ char	id_format[32],nd_format[32];
 
  if ( np==0 )	return(0);
  get_id_format_parameters(ps,np,&id_len,id_format,nd_format);
@@ -321,7 +455,7 @@ int write_photometry(FILE *fw,photstar *ps,int np,char *ofxy,char *ofph,
 	spatialgain *sg,char *nanstring,char *serialstring,int sx,int sy)
 {
  int		i,j,id_len,serial_len,fchar,ok_magn,ok_flux,ok_back;
- char		id_format[8],nd_format[8],buff[128],*f;
+ char		id_format[32],nd_format[32],buff[128],*f;
  double		gain;
  photflux *	pf;
 
@@ -536,7 +670,7 @@ int write_photometry(FILE *fw,photstar *ps,int np,char *ofxy,char *ofph,
 int write_raw_photometry(FILE *fw,photstar *ps,int np,char *nanstring)
 {
  int		i,j,id_len,n;
- char		id_format[8],nd_format[8];
+ char		id_format[32],nd_format[32];
  photflux	*pf;
  double		x,y;
 
@@ -671,133 +805,6 @@ int * create_col_ap_data(char *apcolpar)
 
 }
 
-static int aperture_definition_create_polygon_regular(double **polypoints,int *npolypoint,double requiv,int nvertex,double alpha)
-{
- double	*poly,r;
- int	i;
-
- r=requiv*sqrt((2*M_PI/(double)nvertex)/sin(2*M_PI/(double)nvertex));
- alpha=alpha*M_PI/180.0;
-
- poly=(double *)malloc(sizeof(double)*2*nvertex);
- for ( i=0 ; i<nvertex ; i++ )
-  {	poly[2*i+0]=-r*sin(2*M_PI/(double)nvertex*((double)i+0.5)+alpha);
-	poly[2*i+1]=+r*cos(2*M_PI/(double)nvertex*((double)i+0.5)+alpha);
-	/* fprintf(stderr,"%12g %12g\n",poly[2*i+0],poly[2*i+1]); */
-  }
-
- *polypoints=poly;
- *npolypoint=nvertex;
-
- return(0);
-}
-
-static int aperture_definition_create_polygon_trail(double **polypoints,int *npolypoint,double requiv,int nvertex,double dx,double dy)
-{
- double	*poly,r,alpha;
- int	i;
-
- r=requiv*sqrt((2*M_PI/(double)nvertex)/sin(2*M_PI/(double)nvertex));
- alpha=atan2(dy,dx);
-
- poly=(double *)malloc(sizeof(double)*2*nvertex);
- for ( i=0 ; i<nvertex ; i++ )
-  {	poly[2*i+0]=-r*sin(2*M_PI/(double)nvertex*((double)i+0.5)+alpha)+(i<nvertex/2?-dx/2.0:dx/2.0);
-	poly[2*i+1]=+r*cos(2*M_PI/(double)nvertex*((double)i+0.5)+alpha)+(i<nvertex/2?-dy/2.0:dy/2.0);
-	/* fprintf(stderr,"%12g %12g\n",poly[2*i+0],poly[2*i+1]); */
-  }
-
- *polypoints=poly;
- *npolypoint=nvertex;
-
- return(0);
-}
-
-static int aperture_definition_create_polygon(double **polypoints,int *npolypoint,char *def)
-{
- int	i,n;
- char	**cmd;
- double	*arr;
- while ( *def && *def != '[' )	def++;
- if ( *def=='[' )		def++;
- cmd=tokenize_char_dyn(def,',');
- if ( cmd==NULL )	
-	return(1);
- for ( n=0 ; cmd[n] != NULL ; )	n++;
- if ( n<=0 || 0<(n%2) )
-  {	free(cmd);
-	return(1);
-  }
- *npolypoint=n/2;
- *polypoints=arr=(double *)malloc(sizeof(double)*n);
- for ( i=0 ; i<n ; i++ )
-  {	if ( sscanf(cmd[i],"%lg",&arr[i])<1 )
-	 {	free(arr);
-		free(cmd);
-		return(1);
-	 }
-  }
- free(cmd);
- return(0);
-}
-
-static int aperture_definition_parse(char *apdef,apgeom *ap)
-{
- double	r0,ra,da;
-
- if ( sscanf(apdef,"%lg:%lg:%lg",&r0,&ra,&da)==3 && ( 0.0<r0 && 0.0<=ra && 0.0<da ) )
-  {	ap->apgeom_type=APGEOM_TYPE_CIRCULAR;
-	ap->r0=r0;
-	ap->ra=ra;
-	ap->da=da;
-	return(0);
-  }
- else	
-  {	int	i,n;
-	char	*cmd[8];
-
-	n=tokenize_char(apdef,cmd,':',63);
-	if ( n != 3 )
-		return(1);
-
-	ap->apgeom_type=APGEOM_TYPE_POLYGON;
-
-	ap->r0_poly=NULL;
-	ap->ra_poly=NULL;
-	ap->da_poly=NULL;
-	ap->nr0=0;
-	ap->nra=0;
-	ap->nda=0;
-
-	for ( i=0 ; i<3 ; i++ )
-	 {	double	*polypoints;
-		int	npolypoint;
-
-		double	requiv,alpha,dx,dy;
-		int	nvertex,r;
-
-		if ( sscanf(cmd[i],"Q[%lg,%d,%lg]",&requiv,&nvertex,&alpha)==3 || sscanf(cmd[i],"regular[%lg,%d,%lg]",&requiv,&nvertex,&alpha)==3 )
-			r=aperture_definition_create_polygon_regular(&polypoints,&npolypoint,requiv,nvertex,alpha);
-		else if ( ( sscanf(cmd[i],"T[%lg,%d,%lg,%lg]",&requiv,&nvertex,&dx,&dy)==4 || sscanf(cmd[i],"trail[%lg,%d,%lg,%lg]",&requiv,&nvertex,&dx,&dy)==4 ) && (nvertex%2)==0 )
-			r=aperture_definition_create_polygon_trail(&polypoints,&npolypoint,requiv,nvertex,dx,dy);
-		else if ( ( memcmp(cmd[i],"P[",2)==0 || memcmp(cmd[i],"polygon[",8)==0 ) && cmd[i][strlen(cmd[i])-1]==']' )
-			r=aperture_definition_create_polygon(&polypoints,&npolypoint,cmd[i]);
-	 	else
-			return(1);
-
-		if ( r )
-			return(1);
-
-		if ( i==0 )		ap->r0_poly=polypoints,ap->nr0=npolypoint;
-		else if ( i==1 )	ap->ra_poly=polypoints,ap->nra=npolypoint;
-		else			ap->da_poly=polypoints,ap->nda=npolypoint;
-	 }
-
-	return(0);
-
-  }
-
-}
 
 int create_input_ap_data(char *appar,apgeom **rinaps,int *rninap,int zoom)
 {
