@@ -291,22 +291,6 @@ int add_to_image_weights(fitsimage *img,char **mask,weightlist *wl,double mul)
 
 /*****************************************************************************/
 
-/*****
-typedef struct
-{	fitsimage	*img;
-	char		**mask;
-	weightlist	*wl;
-	int		weightusage;
-	int		is_calc_opt_apert;
-	apgeom		*inaps;
-	int		numap;
-	int		bkhsize;
-	double		gain,sigma;
-	xphotpar	*xpp;
-	double		**bqc;
-} phot_parallel_param;
-*****/
-
 int ringmask_subtract(char **ringmask,int sx,int sy,double x,double y,double r)
 {
  int	i,j,i0,i1,j0,j1;
@@ -388,7 +372,7 @@ char **	ringmask_create(int sx,int sy,photstar *ps,int np,apgeom *inaps,int a,do
 int do_photometry(fitsimage *img,char **mask,
 	photstar *ps,int np,weightlist *wl,int weightusage,int is_calc_opt_apert,
 	apgeom *inaps,int numap,
-	spatialgain *sg,double sigma,xphotpar *xpp)
+	spatialgain *sg,double correlation_length,double sigma,xphotpar *xpp)
 {
  int		i,j,k,jp,sx,sy,r;
  apphotpar	ap;
@@ -756,7 +740,7 @@ int do_photometry(fitsimage *img,char **mask,
 		 { 	wpf->fluxerr=0.0;
 		 }
 		else
-		 {	fluxerr=sqrt((0.0<gain?flux/gain:0.0)+area*(bgsigma*bgsigma)*(1.0+1.0/bgarea)),
+		 {	fluxerr=sqrt((0.0<gain?flux/gain:0.0)+area*(bgsigma*bgsigma*correlation_length*correlation_length)*(1.0+1.0/bgarea)),
 			wpf->fluxerr=fluxerr;
 		 }
 
@@ -858,7 +842,7 @@ int do_photometry(fitsimage *img,char **mask,
 
 int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 	apgeom *inaps,int numap,
-	spatialgain *sg,xphotpar *xpp,kernellist *klist)
+	spatialgain *sg,double correlation_length,xphotpar *xpp,kernellist *klist,int normalize_kernel)
 {
  int		i,j,k,jp,bx,by,sx,sy,r,atot,abad,rtot,rbad,rign,flag;
  apphotpar	ap;
@@ -917,6 +901,8 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 
 	for ( j=0 ; j<n ; j++ )
 	 {	
+		int	apgeom_type;
+
 		wpf=&pf[j];
 
 		wpf->flag=0;
@@ -926,9 +912,16 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 		wpf->rbad=0;
 		wpf->rign=0;
 
-		ap.r0=aps[j].r0,
-		ap.ra=aps[j].ra,
-		ap.da=aps[j].da;
+		apgeom_type=aps[j].apgeom_type;
+
+                ap.r0=aps[j].r0,
+                ap.ra=aps[j].ra,
+                ap.da=aps[j].da;
+
+                ap.r0_poly=aps[j].r0_poly,ap.nr0=aps[j].nr0;
+                ap.ra_poly=aps[j].ra_poly,ap.nra=aps[j].nra;
+                ap.da_poly=aps[j].da_poly,ap.nda=aps[j].nda;
+
 		for ( k=0,jp=-1 ; k<j && jp<0 ; k++ )
 		 {	if ( pf[k].ag.ra==ap.ra && pf[k].ag.da==ap.da )
 			 {	jp=k;break;		}
@@ -942,9 +935,12 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 			abad    =pf[jp].abad;
 			r=0;
 		 }
-		else
+		else if ( apgeom_type==APGEOM_TYPE_CIRCULAR )
 			r=aperture_photometry_back_ring(img->data,mask,sx,sy,cx,cy,&ap,&bgarea,&bgflux,&bgmedian,&bgsigma,&atot,&abad,NULL);
-
+		else if ( apgeom_type==APGEOM_TYPE_POLYGON )
+			r=aperture_photometry_back_polygons(img->data,mask,sx,sy,cx,cy,&ap,&bgarea,&bgflux,&bgmedian,&bgsigma,&atot,&abad,NULL);
+		else
+			r=1;
 
 		if ( r )
 		 {	wpf->flag=MASK_NOBACKGROUND;
@@ -963,8 +959,16 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 				int	rt,rb,ri;
 				if ( xpp->use_biquad )
 					r=aperture_photometry_flux_biquad(bqc,mask,sx,sy,cx+bx,cy+by,ap.r0,&a,&f,&rt,&rb,&ri,xpp->maskignore,NULL);
-				else
+				else if ( apgeom_type==APGEOM_TYPE_CIRCULAR )
 					r=aperture_photometry_flux_circle(img->data,mask,sx,sy,cx+bx,cy+by,ap.r0,&a,&f,NULL,0.0,&rt,&rb,&ri,xpp->maskignore,NULL,0,NULL);
+				else if ( apgeom_type==APGEOM_TYPE_POLYGON )
+					r=aperture_photometry_flux_polygon(img->data,mask,sx,sy,cx+bx,cy+by,ap.r0_poly,ap.nr0,&a,&f,NULL,0.0,&rt,&rb,&ri,xpp->maskignore,NULL);
+				else
+				 {	r=-1;
+					a=f=0.0;
+					rt=rb=ri=0;
+				 }
+					
 				if ( rt>rtot )	rtot=rt;
 				if ( rb>rbad )	rbad=rb;
 				if ( ri>rign )	rign=ri;
@@ -1020,6 +1024,13 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 			 {	for ( bx=0 ; bx<fsize ; bx++ )
 				 {	kflux += kernarr[by][bx];	}
 			 }
+			if ( normalize_kernel && 0<kflux )
+			 {	for ( by=0 ; by<fsize ; by++ )
+				 {	for ( bx=0 ; bx<fsize ; bx++ )
+					 {	kernarr[by][bx] /= kflux;	}
+				 }
+				kflux=1.0;
+			 }
 		 }
 
 		sflux=0.0;
@@ -1064,7 +1075,7 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 			wpf->fluxerr=0.0;
 		 }
 		else
-		 {	fluxerr=sqrt((0.0<gain?flux/gain:0.0)+area*(bgsigma*bgsigma)*(1.0+1.0/bgarea));
+		 {	fluxerr=sqrt((0.0<gain?flux/gain:0.0)+area*(bgsigma*bgsigma*correlation_length*correlation_length)*(1.0+1.0/bgarea));
 			wpf->flux=flux,
 			wpf->fluxerr=fluxerr;
 		 }
@@ -1077,6 +1088,9 @@ int do_subtracted_photometry(fitsimage *img,char **mask,photstar *ps,int np,
 
  if ( bqc != NULL )	tensor_free(bqc);
 
+ tensor_free(kernarr);
+ tensor_free(fluxarr);
+
  return(0);
 }
 
@@ -1309,11 +1323,12 @@ int fprint_fiphot_usage(FILE *fw)
  fprintf(fw,
 "General parameters:\n"
 "\t[-a|--aperture[s] r01[:ra1[:da1]][,r02[:ra2[:da2]][,...]] ]\n"
-"\t[--sky-fit {mean|mode|median,iterations=n,lower=l,upper=u,sigma=s}|\n"
-"\t           |force=<sky>]\n"
+"\t[--sky-fit {mean|mode|median|force=<sky>|\n"
+"\t           |stddev|mad|iterations=n,lower=l,upper=u,sigma=s}]\n"
 "\t[-j|--disjoint-rings | --disjoint-apertures | -x|--disjoint-radius <r>]\n"
 "\t[-z|--zoom <z>] [-k|--spline] [--aperture-mask-ignore <masks>]\n"
 "\t[-g|--gain <gain>[,<polynomial-coeffs>]|auto] [--mag-flux <mag>,<flux>]\n"
+"\t[--correlation-length <background_correlation_length>]\n"
 "\t[-u <subpixelfile> [--normalize]]\n");
  fprintf(fw,
 "Input and output format parameters:\n"
@@ -1470,6 +1485,14 @@ longhelp_entry fiphot_long_help[]=
 	"at the very corners of the image. In such a situation this minimal gain "
 	"is quite useful. " },
 
+ { "--correlation-length <background correlation length>",
+	"Correlation length of the image. This number is used as an estimator "
+	"for background noise in the case of images where the background shows "
+	"correlation. By default, this parameter is unity (1.0). If the image "
+	"is magnified during processing, then one should specify this magnification "
+	"ratio here. For instance, if an image is created with `fitrans --zoom 4`, then "
+	"specify `fiphot --correlation-length 4 ...` as well. " },
+
  { "--mag-flux <mag>,<flux>",
 	"Magnitude - flux conversion level. The specified magnitude will "
 	"be equivalent to the specified flux level. " },
@@ -1577,6 +1600,11 @@ longhelp_entry fiphot_long_help[]=
 	"convolution transformation, which only appropriate if the subtracted "
 	"image is created by a literal arithmetic subtraction (and not "
 	"convolution based subtraction)." },
+ { "--normalize-kernel",
+	"Normalize kernel solution on each point to unity. Useful for "
+	"space-borne photometry with high variations in the PSF shape "
+	"in the cases where the kernel solution is derived empirically "
+	"and there are many variable sources in the field." },
 
  { "Optimal aperture determination:", NULL },
  { "--output-list <output coordinate list file>",
@@ -1706,7 +1734,8 @@ int main(int argc,char *argv[])
  magfitparams	mfp;
  magfitstat	mfs;
  spatialgain	sg;
- int		is_auto_gain=0;
+ double		correlation_length;
+ int		is_auto_gain=0,normalize_kernel;
 
  progbasename=strrchr(argv[0],'/');
  if ( progbasename != NULL )	progbasename++;
@@ -1727,8 +1756,9 @@ int main(int argc,char *argv[])
 
  skyfitpar=NULL;
  xpp.bgm.type=BGTYPE_MEDIAN;
- xpp.bgm.rejniter=2,
- xpp.bgm.rejlower=xpp.bgm.rejupper=2.0;
+ xpp.bgm.scatter=BGSCATTER_STDDEV;
+ xpp.bgm.rejniter=0,
+ xpp.bgm.rejlower=xpp.bgm.rejupper=3.0;
  xpp.maskignore=MASK_SATURATED|MASK_HOT;
  xpp.use_biquad=0;
  xpp.wconfdist=0.0;
@@ -1748,6 +1778,8 @@ int main(int argc,char *argv[])
  col.colid=col.colmag=col.colcol=-1;
  
  sg.vmin=0.0;
+ correlation_length=1.0;
+ normalize_kernel=0;
 
  zoom=1;
 
@@ -1770,8 +1802,10 @@ int main(int argc,char *argv[])
 	"-r|--reference:%s",&refname,
 	"-s|--input-subtracted:%s",&subname,
 	"-K|--input-kernel:%s",&subkernelfile,
+	"--normalize-kernel:%f",&normalize_kernel,
 	"-d|--skysigma|--sky-noise|--skynoise:%g",&skysigma,
 	"-g|--gain|--gain-poly|--gainpoly:%s",&gainstr,
+	"--correlation-length:%g",&correlation_length,
 	"--gain-vmin|--gainvmin:%g",&sg.vmin,
 	"-f|--fwhm:%g",&fwhm,
 	"--mag-flux:%g,%g",&mf0.magnitude,&mf0.intensity,
@@ -1908,12 +1942,15 @@ int main(int argc,char *argv[])
 		"mean:"  SNf(BGTYPE_MEAN)  ,&xpp.bgm.type,
 		"median:"SNf(BGTYPE_MEDIAN),&xpp.bgm.type,
 		"mode:"  SNf(BGTYPE_MODE)  ,&xpp.bgm.type,
+		"mad:"   SNf(BGSCATTER_MAD)   ,&xpp.bgm.scatter,
+		"stddev:"SNf(BGSCATTER_STDDEV),&xpp.bgm.scatter,
 		"iterations:%d",&xpp.bgm.rejniter,
 		"lower:%g",&xpp.bgm.rejlower,
 		"upper:%g",&xpp.bgm.rejupper,
 		"sigma:%g%f",&sigma,&is_sigma,
 		"force:%g%f",&xpp.sky,&xpp.use_sky,
 		NULL);
+
 	if ( is_sigma )	xpp.bgm.rejlower=xpp.bgm.rejupper=sigma;
   }
 		
@@ -2083,7 +2120,7 @@ int main(int argc,char *argv[])
 
 	if ( is_auto_gain )
 		get_gain_info_from_header(&sg,&img->header);
-	do_photometry(&img->i,mask,ps,np,wl,weightusage,is_calc_opt_apert,inaps,numap,&sg,sigma,&xpp);
+	do_photometry(&img->i,mask,ps,np,wl,weightusage,is_calc_opt_apert,inaps,numap,&sg,correlation_length,sigma,&xpp);
 
 	fits_free(img);
 
@@ -2174,7 +2211,7 @@ int main(int argc,char *argv[])
 		logmsg(is_verbose,"Performing reference photometry ... ");
 		if ( is_auto_gain )
 			get_gain_info_from_header(&sg,&refimg->header);
-		do_photometry(&refimg->i,mask,ps,np,wl,weightusage,0,inaps,numap,&sg,sigma,&xpp);
+		do_photometry(&refimg->i,mask,ps,np,wl,weightusage,0,inaps,numap,&sg,correlation_length,sigma,&xpp);
 		logmsg(is_verbose,"done.\n");
 		for ( i=0 ; i<np ; i++ )
 		 {	ps[i].rfflux=ps[i].fluxes;
@@ -2228,7 +2265,7 @@ int main(int argc,char *argv[])
 		get_gain_info_from_header(&sg,&subimg->header);
 
 	logmsg(is_verbose,"Photometry on subtracted image '%s' ... ",subname);
-	do_subtracted_photometry(&subimg->i,submask,ps,np,inaps,ninap,&sg,&xpp,kl);
+	do_subtracted_photometry(&subimg->i,submask,ps,np,inaps,ninap,&sg,correlation_length,&xpp,kl,normalize_kernel);
 	logmsg(is_verbose,"done.\n");
 
 	fits_mask_free(submask);
