@@ -145,17 +145,25 @@ int fprint_fitrans_usage(FILE *fw)
 "\t[[-i|--input] <in>[<F>] [--frame <F>] [-M|--input-mask <mask>]]\n"
 "\t[-C|--comment] [-V|--verbose] [-o|--output <output>]\n");
  fprintf(fw,
-"General (analytical) transformation:\n"
+"General (analytical) transformations and shifting:\n"
 "\t[-T <transform-file>|-t <transform> [-m|-l|-c|-k] [--reverse]] \n"
+"\t[-e|--shift <dx>,<dy>]\n");
+ fprintf(fw,
 "Layer extraction or slicing data cubeimages into disctint layers:\n"
 "\t[-y|--layer <layer>]\n"
 "\t[-x|--explode <basename> [-y|--first-layer <n>]]\n");
+ fprintf(fw,
+"Multiple stamp extraction:\n"
+"\t[-u|--cut <basename> -p|--step <dx>,<dy> -w|--count <nx>,<ny>\n"
+"\t -f|--offset <ox>,<oy> -s|--size <sx>,<sy>]\n");
 
  fprintf(fw,
-"Zooming/shrinking, shifting:\n"
-"\t[-z|--zoom <factor>] [-g|--magnify <factor>]\n"
-"\t[-r|--shrink <factor> [-d|--median] [--optimistic-masking]]\n"
-"\t[-e|--shift <dx>,<dy>]\n");
+"Zooming, shrinking, interleaving and repetitive expanding:\n"
+"\t[-z|--zoom <factor>|--zoom-xy <x>,<y>]\n"
+"\t[-g|--magnify <factor>|--magnify-xy <x>,<y>]\n"
+"\t[-r|--shrink <f>|--shrink-xy <x>,<y> [--median] [--optimistic-masking]]\n"
+"\t[--interleave-xy <x>,<y> [--median] [--optimistic-masking]]\n"
+"\t[--repetitive-xy <x>,<y>]\n");
 
  fprintf(fw,
 "Noise estimation:\n"
@@ -249,18 +257,21 @@ longhelp_entry fitrans_long_help[]=
 	"Y coordinate." },
 
  { "Other simple spatial geometric transformations:", NULL },
- { "-z, --zoom <factor>",
+ { "-z, --zoom <factor>, --zoom-xy <x>,<y>",
 	"Zoom the image by the given (integer) factor, involving a biquadratic "
 	"subpixel-level interpolation and therefore exact flux conservation. " },
- { "-r, --shrink <factor>",
-	"Shrink the image by the given (integer) factor. " },
+ { "-r, --shrink <factor>, --shrink-xy <x>,<y>",
+	"Shrink the image by the given (integer) factor(s)."},
  { "-d, --median",
 	"Use a median-based averaging during the shrinking operation. " },
+ { "-v, --truncated-mean <Nr>",
+	"Compute a truncated mean during the shrinking operation by rejecting the <Nr> "
+ 	"number of lower and upper points. " },
  { "--optimistic-masking",
 	"Imply some optimism during the shrinking operation: masked pixels "
 	"are ignored during the averaging process and the final mask will "
 	"be computed in a complement manner. " },
- { "-g, --magnify <factor>",
+ { "-g, --magnify <factor>, --magnify-xy <x>,<y>",
 	"Same as zooming the image but there is no subpixel-level interpolation." },
 
  { "Large-scale image smoothing:", NULL },
@@ -308,6 +319,33 @@ longhelp_entry fitrans_long_help[]=
 	"Use the specified value for the first layer index. The subsequent "
 	"layer indices are incremented normally. By default, the index of "
 	"the first data cube layer is 0. " },
+
+ { "Multiple stamp extraction:", NULL },
+ { "-c, --cut <basename>",
+	"Extract stamps with a size defined by the -s|--size argument. "
+	"The stamps are extracted on a grid, defined by the -p|--step and "
+	"-w|--count arguments where the stamp grid offset is defined by the "
+	"-f|--offset argument. The <basename> should contain two spefic "
+	"printf-like elements of %x and %y which are replaced by the "
+	"corresponding stamp indices in the X and Y directions. In order to "
+	"have some sort of pretty-printing in the file names, the %x and %y "
+	"parts may have the form of %.<N>x and %.<N>y. Note that despite of "
+	"the %x convention, both %x and %y are printed in decimal system. " },
+ { "-w, --count <count_x>,<count_y>",
+	"Number of stamps to be extracted in X and Y directions, "
+	"respectively. " },
+ { "-s, --size <sx>,<sy>",
+	"The size of the output image stamps. " },
+ { "-p, --step <step_x>,<step_y>",
+	"Step size between the stamps in the units of pixels and in the X and "
+	"Y directions, respectively. Note that if <step_x> is smaller than "
+	"<sx> (or similarly, if <step_y> is smaller than <sy>), then the "
+	"output stamps will overlap. This might also be intentional for "
+	"various applications. " },
+ { "-f, --offset <x>,<y>", 
+	"Zero-point coordinate of the lower-left pixel of the lower-left "
+	"output stamp (having an index of %x=0 and %y=0) in the frame of the "
+	"input image." },
  
  { NULL, NULL }
 };
@@ -841,6 +879,39 @@ int interpolate_image_bicubic(double **c,int sx,int sy,char **mask,double fx,dou
  return(flag);
 }
 
+double apply_transformation_get_sqjacobi(fitsimage *outimg,int ofx,int ofy,transformation *tf,int is_invert)
+{
+ double	x,y;
+ int	nsx,nsy;
+ double	*jxx,*jxy,*jyx,*jyy,cjxx,cjxy,cjyx,cjyy,cj,sqjacobi;
+
+ nsx=outimg->sx;
+ nsy=outimg->sy;
+
+ transformation_get_jacobi(tf,&jxx,&jxy,&jyx,&jyy);
+ 
+ x=(double)nsx/2.0-ofx;
+ y=(double)nsy/2.0-ofy;
+ 
+ cjxx=eval_2d_poly(x,y,tf->order-1,jxx,tf->ox,tf->oy,tf->scale);
+ cjxy=eval_2d_poly(x,y,tf->order-1,jxy,tf->ox,tf->oy,tf->scale);
+ cjyx=eval_2d_poly(x,y,tf->order-1,jyx,tf->ox,tf->oy,tf->scale);
+ cjyy=eval_2d_poly(x,y,tf->order-1,jyy,tf->ox,tf->oy,tf->scale);
+
+ if ( ! is_invert )	cj=cjxx*cjyy-cjxy*cjyx;
+ else			cj=1.0/(cjxx*cjyy-cjxy*cjyx);
+
+ sqjacobi=sqrt(fabs(cj));
+ if ( 0.0<sqjacobi )	sqjacobi=1.0/sqjacobi;
+
+ if ( jyy != NULL )	free(jyy);
+ if ( jyx != NULL )	free(jyx);
+ if ( jxy != NULL )	free(jxy);
+ if ( jxx != NULL )	free(jxx);
+
+ return(sqjacobi);
+}
+
 int apply_transformation_polynomial_interpolate
 	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
 	int ofx,int ofy,transformation *tf,int is_spline,int is_invert)
@@ -1047,7 +1118,7 @@ int apply_transformation
 
 int zoom_image
 	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
-	int ofx,int ofy,int scale)
+	int ofx,int ofy,int scale_x,int scale_y)
 {
  int	sx,sy,nsx,nsy;
  int	i,j,k,l,m,ii,jj;
@@ -1061,45 +1132,45 @@ int zoom_image
  nsy=outimg->sy;
  if ( sx==0 || sy==0 || nsx==0 || nsy==0 )	return(0);
 
- wret=(double **)tensor_alloc_2d(double,scale,scale);
+ wret=(double **)tensor_alloc_2d(double,scale_x,scale_y);
  bqc =(double **)tensor_alloc_2d(double,2*sx+1,2*sy+1);
  logmsg(is_verbose,"Calculating biquadratic coefficients... ");
  biquad_coeff(img->data,sx,sy,bqc,mask);
  logmsg(is_verbose,"done.\n");
 
  logmsg(is_verbose==1,"Zooming... ");
- for ( i=0 ; i<nsy/scale ; i++ )
+ for ( i=0 ; i<nsy/scale_y ; i++ )
   {	ii=i+ofy;
-	for ( j=0 ; j<nsx/scale ; j++ )
+	for ( j=0 ; j<nsx/scale_x ; j++ )
 	 {	jj=j+ofx;
 		m=0;
 		if ( ii<0 || jj<0 || ii>sy-1 || jj>sx-1 )
-		 {	for ( k=0 ; k<scale ; k++ )
-			 {  for ( l=0 ; l<scale ; l++ )
+		 {	for ( k=0 ; k<scale_y ; k++ )
+			 {  for ( l=0 ; l<scale_x ; l++ )
 			     {	wret[k][l]=0.0;			}
 			 }
 			m|=MASK_OUTER;
 		 }
 		else if ( mask[ii][jj] )
-		 {	for ( k=0 ; k<scale ; k++ )
-			 {  for ( l=0 ; l<scale ; l++ )
+		 {	for ( k=0 ; k<scale_y ; k++ )
+			 {  for ( l=0 ; l<scale_x ; l++ )
 			     {	wret[k][l]=0.0;			}
 			 }
 			m|=mask[ii][jj];
 		 }
 		else 
-	 	 {	biquad_isc_int_block_subpixels(bqc,jj,ii,scale,scale,wret);
+	 	 {	biquad_isc_int_block_subpixels(bqc,jj,ii,scale_x,scale_y,wret);
 			m=0;
 		 }
 
-		for ( k=0 ; k<scale ; k++ )
-		 {  for ( l=0 ; l<scale ; l++ )
-		     {	outimg->data[i*scale+k][j*scale+l]=wret[k][l];
-			outmask[i*scale+k][j*scale+l]=m;
+		for ( k=0 ; k<scale_y ; k++ )
+		 {  for ( l=0 ; l<scale_x ; l++ )
+		     {	outimg->data[i*scale_y+k][j*scale_x+l]=wret[k][l];
+			outmask[i*scale_y+k][j*scale_x+l]=m;
 		     }
 		 }
 	 }
-	logmsg(is_verbose>=2,"\rZooming... (%d/%d)... ",i+1,nsy/scale);
+	logmsg(is_verbose>=2,"\rZooming... (%d/%d)... ",i+1,nsy/scale_y);
   }
  logmsg(is_verbose,"done.\n");
 
@@ -1111,7 +1182,7 @@ int zoom_image
 
 int zoom_raw_image
 	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
-	int ofx,int ofy,int scale)
+	int ofx,int ofy,int scale_x,int scale_y)
 {
  int	sx,sy,nsx,nsy;
  int	i,j,k,l,m,ii,jj;
@@ -1125,11 +1196,11 @@ int zoom_raw_image
  nsy=outimg->sy;
  if ( sx==0 || sy==0 || nsx==0 || nsy==0 )	return(0);
 
- sfactor=1.0/(double)(scale*scale);
+ sfactor=1.0/(double)(scale_x*scale_y);
 
- for ( i=0 ; i<nsy/scale ; i++ )
+ for ( i=0 ; i<nsy/scale_y ; i++ )
   {	ii=i+ofy;
-	for ( j=0 ; j<nsx/scale ; j++ )
+	for ( j=0 ; j<nsx/scale_x ; j++ )
 	 {	jj=j+ofx;
 		m=0;
 		if ( ii<0 || jj<0 || ii>sy-1 || jj>sx-1 )
@@ -1145,10 +1216,10 @@ int zoom_raw_image
 			m=0;
 		 }
 
-		for ( k=0 ; k<scale ; k++ )
-		 {  for ( l=0 ; l<scale ; l++ )
-		     {	outimg->data[i*scale+k][j*scale+l]=wr;
-			outmask[i*scale+k][j*scale+l]=m;
+		for ( k=0 ; k<scale_y ; k++ )
+		 {  for ( l=0 ; l<scale_x ; l++ )
+		     {	outimg->data[i*scale_y+k][j*scale_x+l]=wr;
+			outmask[i*scale_y+k][j*scale_x+l]=m;
 		     }
 		 }
 	 }
@@ -1157,11 +1228,51 @@ int zoom_raw_image
  return(0); 
 }
 
+int repetitive_image
+	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
+	int ofx,int ofy)
+{
+ int	sx,sy,nsx,nsy;
+ int	i,j,ii,jj;
+
+ if ( img==NULL || outimg==NULL )	return(1);
+ if ( mask==NULL || outmask==NULL )	return(1);
+ sx=img->sx,
+ sy=img->sy;
+ nsx=outimg->sx;
+ nsy=outimg->sy;
+ if ( sx==0 || sy==0 || nsx==0 || nsy==0 )	return(0);
+
+ if ( ofx<0 )	ofx+=((-ofx)/nsx+2)*nsx;
+ if ( ofy<0 )	ofy+=((-ofy)/nsy+2)*nsy;
+ ofx=ofx%nsx;
+ ofy=ofy%nsy;
+
+ for ( i=0 ; i<nsy ; i++ )
+  {	ii=(i+ofy)%sy;
+	for ( j=0 ; j<nsx ; j++ )
+	 {	jj=(j+ofx)%sx;
+
+		outimg->data[i][j]=img->data[ii][jj];
+		outmask[i][j]=mask[ii][jj];
+	 }
+  }
+
+ return(0); 
+}
+
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+typedef struct
+ {	int	mode_median;		/* 0: normal mean, 1: median 				*/
+	int	mode_truncated_mean;	/* 0: normal mean: 0<: reject lower & upper points	*/
+	int	mode_average_mask;	/* handling masks					*/
+ } shrink_mode;
+
 int shrink_image
 	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
-	int ofx,int ofy,int scale,int do_median,int do_avg_mode)
+	int ofx,int ofy,int scale_x,int scale_y,shrink_mode *mode)
 {
  int	sx,sy,nsx,nsy;
  int	i,j,k,l,ii,jj,m,t;
@@ -1175,35 +1286,40 @@ int shrink_image
  nsy=outimg->sy;
  if ( sx==0 || sy==0 || nsx==0 || nsy==0 )	return(0);
 
- if ( do_median )
-	medarr=(double *)malloc(sizeof(double)*(scale*scale));
+ if ( mode->mode_median || 0<mode->mode_truncated_mean )
+	medarr=(double *)malloc(sizeof(double)*(scale_x*scale_y));
  else
 	medarr=NULL;
 
  for ( i=0 ; i<nsy ; i++ )
-  {	ii=i*scale+ofy;
+  {	ii=i*scale_y+ofy;
 	for ( j=0 ; j<nsx ; j++ )
-	 {	jj=j*scale+ofx;
-		if ( ii<0 || jj<0 || ii>sy-scale || jj>sx-scale )
+	 {	jj=j*scale_x+ofx;
+		if ( ii<0 || jj<0 || ii>sy-scale_y || jj>sx-scale_x )
 		 {	outimg->data[i][j]=0.0;
 			outmask[i][j]=MASK_OUTER;
 		 }
 		else if ( medarr != NULL )
 		 {	m=0;
-			if ( do_avg_mode==0 )
-			 {	for ( k=0,t=0 ; k<scale ; k++ )
-				 {  for ( l=0 ; l<scale ; l++ )
+			if ( mode->mode_average_mask==0 )
+			 {	for ( k=0,t=0 ; k<scale_y ; k++ )
+				 {  for ( l=0 ; l<scale_x ; l++ )
 				     {	m|=mask[ii+k][jj+l];
-					medarr[t]=img->data[ii+k][jj+l];
-					t++;
+					if ( ! mask[ii+k][jj+l] )
+					 {	medarr[t]=img->data[ii+k][jj+l];
+						t++;
+					 }
 				     }
 				 }
 				outmask[i][j]=m;
-				outimg->data[i][j]=median(medarr,t)*(double)(scale*scale);
+				if ( mode->mode_median )
+					outimg->data[i][j]=median(medarr,t)*(double)(scale_x*scale_y);
+				else if ( 0<mode->mode_truncated_mean )
+					outimg->data[i][j]=truncated_mean(medarr,t,mode->mode_truncated_mean)*(double)(scale_x*scale_y);
 			 }
-			else if ( do_avg_mode==1 )
-			 {	for ( k=0,t=0 ; k<scale ; k++ )
-				 {  for ( l=0 ; l<scale ; l++ )
+			else if ( mode->mode_average_mask==1 )
+			 {	for ( k=0,t=0 ; k<scale_y ; k++ )
+				 {  for ( l=0 ; l<scale_x ; l++ )
 				     {	m|=~mask[ii+k][jj+l];
 					if ( ! mask[ii+k][jj+l] )
 					 {	medarr[t]=img->data[ii+k][jj+l];
@@ -1212,7 +1328,11 @@ int shrink_image
 				     }
 				 }
 				outmask[i][j]=(~m)&MASK_ALL;
-				outimg->data[i][j]=median(medarr,t)*(double)(scale*scale);
+				if ( mode->mode_median )
+					outimg->data[i][j]=median(medarr,t)*(double)(scale_x*scale_y);
+				else if ( 0<mode->mode_truncated_mean )
+					outimg->data[i][j]=truncated_mean(medarr,t,mode->mode_truncated_mean)*(double)(scale_x*scale_y);
+					
 			 }
 			else
 			 {	outmask[i][j]=MASK_ALL;
@@ -1221,8 +1341,8 @@ int shrink_image
 		 }
 		else
 		 {	w=0.0;m=0;
-			for ( k=0 ; k<scale ; k++ )
-			 {  for ( l=0 ; l<scale ; l++ )
+			for ( k=0 ; k<scale_y ; k++ )
+			 {  for ( l=0 ; l<scale_x ; l++ )
 			     {	m|=mask[ii+k][jj+l];
 				w+=img->data[ii+k][jj+l];
 			     }
@@ -1239,6 +1359,79 @@ int shrink_image
  return(0);
 }
 
+int interleave_image
+	(fitsimage *img,char **mask,fitsimage *outimg,char **outmask,
+	int ofx,int ofy,shrink_mode *mode)
+{
+ int	sx,sy,nsx,nsy;
+ int	i,j,k,l,ii,jj,m,t,bwx,bwy;
+ double	*medarr;
+
+ if ( img==NULL || outimg==NULL )	return(1);
+ if ( mask==NULL || outmask==NULL )	return(1);
+ sx=img->sx,
+ sy=img->sy;
+ /* note: heare nsx and nsy equal to interleave_x and interleave_y, respecitvely: */
+ nsx=outimg->sx;
+ nsy=outimg->sy;
+ if ( sx==0 || sy==0 || nsx==0 || nsy==0 )	return(0);
+
+ bwx=(sx+nsx-1)/nsx;
+ bwy=(sy+nsy-1)/nsy;
+
+ if ( ofx<0 )	ofx+=((-ofx)/nsx+2)*nsx;
+ if ( ofy<0 )	ofy+=((-ofy)/nsy+2)*nsy;
+ ofx=ofx%nsx;
+ ofy=ofy%nsy;
+
+ medarr=(double *)malloc(sizeof(double)*(bwx*bwy));
+
+ for ( i=0 ; i<nsy ; i++ )
+  {	int	i0;
+	i0=(i+ofy)%nsy;
+	
+	for ( j=0 ; j<nsx ; j++ )
+	 {	int	j0;
+		j0=(j+ofx)%nsx;
+
+		t=0;
+		m=0;
+		for ( k=0; k<bwy ; k++ )
+		 {	ii=i0+k*nsy;
+			if ( sy<=ii )
+				continue;
+			for ( l=0; l<bwx ;l++ )
+			 {	jj=j0+l*nsx;
+				if ( sx<=jj )
+					continue;
+				medarr[t]=img->data[ii][jj];
+				if ( ! mode->mode_average_mask )
+					m|= mask[ii][jj];
+				else	
+					m|=~mask[ii][jj];
+				if ( mode->mode_average_mask && mask[ii][jj] )
+					continue;
+				t++;
+			 }
+		 }
+		if ( mode->mode_median )
+		 {	outimg->data[i][j]=median(medarr,t);	}
+		else
+		 {	outimg->data[i][j]=mean(medarr,t);	}
+		if ( ! mode->mode_average_mask )
+			outmask[i][j]=m;
+		else
+			outmask[i][j]=(~m)&MASK_ALL;
+
+	 }
+  }
+
+ if ( medarr != NULL )
+	free(medarr);
+
+ return(0);
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 int scatter_image
@@ -1317,33 +1510,51 @@ int trim_image
 
 int main(int argc,char *argv[])
 {
- FILE		*fw,*fr;
  int		i,is_help,is_flip_x,is_flip_y,is_invert,is_shift;
- fits		*img,*outimg;
- int		ofx,ofy,nsx,nsy,frameno,layer,do_median,do_avg_mode;
- char		*outimgfile,*inimgfile,*transfile,*transparam,*basename,
-		**inmasklist,*inweightfile;
- int		zratio,gratio,sratio,bqscatt,int_method;
- char		**mask,**outmask,*fdpstring;
+ int		ofx,ofy,nsx,nsy,frameno,layer;
+ shrink_mode	sh_mode;
+ char		*outimgfile,*inimgfile,*transfile,*transparam;
+ char		**inmasklist,*inweightfile;
+ int		zoom_ratio,magnify_ratio,shrink_ratio;
+ int		zratio_x,zratio_y,gratio_x,gratio_y,sratio_x,sratio_y;
+ int		interleave_x,interleave_y,repetitive_x,repetitive_y;
+ int		bqscatt,int_method;
+ char		*fdpstring;
  char		*smoothparam;
  char		*explode_basename;
+ char		*cut_basename;
+ int		stepx,stepy,ncutx,ncuty;
  transformation	trf_data,*trf=&trf_data;
  fitsdataparam	fdp;
  weightlist	wl_data,*wl;
  smooth		sp;
- double		shift_dx,shift_dy;
+ double		shift_dx,shift_dy,sqjacobi;
 
  progbasename=strrchr(argv[0],'/');
  if ( progbasename != NULL )	progbasename++;
  else				progbasename=argv[0];
 
  outimgfile=inimgfile=transfile=inweightfile=transparam=NULL;inmasklist=NULL;
- is_comment=is_verbose=is_help=0;zratio=sratio=gratio=bqscatt=0;
+ is_comment=is_verbose=is_help=0;
+
+ zoom_ratio=shrink_ratio=magnify_ratio=0;
+ zratio_x=zratio_y=0;
+ gratio_x=gratio_y=0;
+ sratio_x=sratio_y=0;
+ interleave_x=interleave_y=0;
+ repetitive_x=repetitive_y=0;
+
+ bqscatt=0;
  ofx=ofy=is_flip_x=is_flip_y=nsx=nsy=0;int_method=is_invert=0;
- do_median=do_avg_mode=0;
+
+ sh_mode.mode_median=0;
+ sh_mode.mode_truncated_mean=0;
+ sh_mode.mode_average_mask=0;
+
  frameno=0;
  smoothparam=NULL;
- explode_basename=NULL;
+ cut_basename=explode_basename=NULL;
+ stepx=stepy=ncutx=ncuty=0;
 
  is_shift=0;
  shift_dx=shift_dy=0.0;
@@ -1373,11 +1584,17 @@ int main(int argc,char *argv[])
 	"-c:%SN2f",&int_method, 	/* interpolate + spline		*/
 	"-k:%SN3f",&int_method,		/* integrate   + spline		*/
 	"-M|--input-mask:%t",&inmasklist,
-	"-z|--zoom:%d",&zratio,
-	"-g|--magnify:%d",&gratio,
-	"-r|--shrink:%d",&sratio, 
-	"-d|--median:%f",&do_median,
-	"--optimistic-masking:%SN1f",&do_avg_mode,
+	"-z|--zoom:%d",&zoom_ratio,
+	"-g|--magnify:%d",&magnify_ratio,
+	"-r|--shrink:%d",&shrink_ratio, 
+	"--zoom-xy:%d,%d",&zratio_x,&zratio_y,
+	"--magnify-xy:%d,%d",&gratio_x,&gratio_y,
+	"--shrink-xy:%d,%d",&sratio_x,&sratio_y,
+	"--interleave-xy:%d,%d",&interleave_x,&interleave_y,
+	"--repetitive-xy:%d,%d",&repetitive_x,&repetitive_y,
+	"-d|--median:%f",&sh_mode.mode_median,
+	"-v|--truncated-mean:%d",&sh_mode.mode_truncated_mean,
+	"--optimistic-masking:%SN1f",&sh_mode.mode_average_mask,
 	"-n|--noise:%f",&bqscatt,
 	"-a|--smooth:%s",&smoothparam,
 	"-f|--offset:%d,%d",&ofx,&ofy,
@@ -1385,6 +1602,9 @@ int main(int argc,char *argv[])
 	"-e|--shift:%f%g,%g",&is_shift,&shift_dx,&shift_dy,
 	"-y|--layer|--first-layer:%d",&layer,
 	"-x|--explode:%s",&explode_basename,
+	"-u|--cut:%s",&cut_basename,
+	"-p|--step:%d,%d",&stepx,&stepy,
+	"-w|--count:%d,%d",&ncutx,&ncuty,
 	"--flip-x:%f",&is_flip_x,
 	"--flip-y:%f",&is_flip_y,
 	"--flip-xy|--flip-yx:%f%f",&is_flip_x,&is_flip_y,
@@ -1414,6 +1634,10 @@ int main(int argc,char *argv[])
 	return(0);
   }
 
+ if ( 0<zoom_ratio )	zratio_x=zratio_y=zoom_ratio;
+ if ( 0<shrink_ratio )	sratio_x=sratio_y=shrink_ratio;
+ if ( 0<magnify_ratio )	gratio_x=gratio_y=magnify_ratio;
+
  is_invert=!is_invert;	/* negate to be self-consistent with grtrans!	*/
 			/* [inserted: 2006.05.09]			*/
 
@@ -1469,7 +1693,9 @@ int main(int argc,char *argv[])
   }
 
  if ( inweightfile != NULL )
-  {	fits	*img;
+  {	FILE	*fr;
+	fits	*img;
+	
 	if ( (fr=fopenread(inweightfile))==NULL )
 	 {	fprint_error("unable to open input weight file '%s'.",inweightfile);
 		return(1);
@@ -1489,9 +1715,13 @@ int main(int argc,char *argv[])
  else
 	wl=NULL;
 
+ /* datacube -> layer extraction */
  if ( explode_basename != NULL )
   {	int		i,l,nlayer,sx,sy;
 	fitsimage	*fi;
+	char		*basename;
+	FILE		*fr;
+	fits		*img;
 
 	if ( ! format_check_if_formatted(explode_basename,"dxXio") ) 
 	 {	fprint_error("invalid explode basename template '%s' (perhaps '%%d' is missing)",explode_basename);
@@ -1567,202 +1797,310 @@ int main(int argc,char *argv[])
 	fcloseread(fr);
 	return(0);
   }
+ /* image -> multiple stamp extraction: */
+ else if ( cut_basename != NULL )
+  {
+ 	char	*basename;
+	FILE	*fr;
+	fits	*img;
+	char	buff[128];
+	int	cx,cy;
+	char	**mask;
 
- basename=fits_basename(inimgfile,&frameno);
- if ( (fr=fopenread(basename))==NULL )
-  {	fprint_error("unable to open input file '%s'.",basename);
-	return(1);
-  }
- img=fits_read_frame_to_image(fr,frameno);
- fclose(fr);
- if ( img==NULL )
-  {	fprint_error("unable to interpret input as FITS data.");
-	return(1); 
-  }
- if ( img->i.dim != 2 && layer<0 )
-  {	fprint_error("image dimension differs from 2 and no layer has been specified.");
-	return(1); 
-  }
- else if ( img->i.dim == 3 && layer>=0 && layer<img->i.naxis[2] )
-  {	fitsimage	fi;
-	int		i,sx,sy;
-	double		***dcube;
-	sx=img->i.naxis[0];
-	sy=img->i.naxis[1];
-	fits_image_alloc(&fi,sx,sy);
-	fi.bit=img->i.bit;
-	fi.curr=img->i.curr;
-	fi.read=img->i.read;
-	dcube=(double ***)img->i.vdata;
-	for ( i=0 ; i<sy ; i++ )
-	 {	memcpy(fi.data[i],dcube[layer][i],sizeof(double)*sx);	}
-	fits_image_free(&img->i);
-	img->i=fi;
-	fits_image_set_params(&img->header,&img->i);
-  }
- fits_rescale(img);
- 
- logmsg(is_verbose,"[%d,%d]\n",img->i.sx,img->i.sy);
-
-/* Create the mask for the valid pixels: */
- mask=fits_mask_read_from_header(&img->header,img->i.sx,img->i.sy,NULL);
- if ( inmasklist != NULL )
-  {	if ( join_masks_from_files(mask,img->i.sx,img->i.sy,inmasklist) )
-	 {	fprint_error("unable to read one of the input mask files");
+	basename=fits_basename(inimgfile,&frameno);
+	if ( (fr=fopenread(basename))==NULL )
+	 {	fprint_error("unable to open input file '%s'.",basename);
 		return(1);
 	 }
-  }
- fits_mask_mark_nans(&img->i,mask,MASK_NAN);
-
- if ( nsx<0 )	is_flip_x=!is_flip_x,nsx=-nsx;
- if ( nsy<0 )	is_flip_y=!is_flip_y,nsy=-nsy;
-
- if ( nsx==0 && nsy==0 )
-  {	if ( zratio>1 )
-		nsx=img->i.sx*zratio,
-		nsy=img->i.sy*zratio;
-	else if ( gratio>1 )
-		nsx=img->i.sx*gratio,
-		nsy=img->i.sy*gratio;
-	else if ( sratio>1 )
-		nsx=(img->i.sx+sratio-1)/sratio,
-		nsy=(img->i.sy+sratio-1)/sratio;
-	else
-		nsx=img->i.sx,
-		nsy=img->i.sy;
-  }
-
-/* Create new FITS image for the output: */
- outimg=fits_duplicate_empty(img);
- if ( nsx != img->i.sx || nsy != img->i.sy )
-  {	fits_free_image(outimg);
-	fits_alloc_image(outimg,nsx,nsy);
-	fits_reset_image(outimg);
-	fits_set_header_integer(outimg,"NAXIS1",FITS_SH_FIRST,nsx,NULL);
-	fits_set_header_integer(outimg,"NAXIS2",FITS_SH_FIRST,nsy,NULL);
-  }
- outmask=fits_mask_create_empty(nsx,nsy);
-
-/* Read transformation file, if it has been specified in the command line: */
- if ( transfile != NULL )
-  {	FILE	*ft;
-	ft=fopenread(transfile);
-	if ( ft==NULL )	
-	 {	fprint_error("unable to open input transformation file '%s'",transfile);
-		return(1);
+	img=fits_read_frame_to_image(fr,frameno);
+	fclose(fr);
+	if ( img==NULL )
+	 {	fprint_error("unable to interpret input as FITS data.");
+		return(1); 
 	 }
-	i=transformation_read_data(ft,trf);
-	if ( i )
-	 {	fprint_error("unable to parse the contents of input transformation file '%s'",transfile);
-		return(1);
+	else if ( img->i.dim != 2 )
+	 {	fprint_error("image dimension differs from 2.");
+		return(1); 
 	 }
-	fcloseread(ft);
-  }
- else if ( transparam != NULL )
-  {	i=transformation_parse_params(transparam,trf);
-	if ( i )		
-	 {	fprint_error("unable to parse transformation string '%s'",transparam);
-		return(1);
+	fits_rescale(img);
+
+	/* Create the mask for the valid pixels: */
+	mask=fits_mask_read_from_header(&img->header,img->i.sx,img->i.sy,NULL);
+
+	for ( cy=0 ; cy<ncuty ; cy++ )
+	 {  for ( cx=0 ; cx<ncutx ; cx++ )
+	     {	char	filename[128];
+		fits	*outimg;
+		char	**outmask;
+		FILE	*fw;
+
+		outimg=fits_create();
+		outmask=fits_mask_create_empty(nsx,nsy);
+
+		fits_set_standard(outimg,NULL);
+		fits_alloc_image(outimg,nsx,nsy);
+		outimg->i.bit = img->i.bit;
+		outimg->i.curr.bscale=img->i.curr.bscale;
+		outimg->i.curr.bzero =img->i.curr.bzero;
+		outimg->i.read.bscale=img->i.read.bscale;
+		outimg->i.read.bzero =img->i.read.bzero;
+		fits_set_image_params(outimg);
+		sprintf(buff,"Created by fitrans, version: %s",FITSH_FITRANS_VERSION);
+		fits_set_origin(outimg,buff,NULL);
+		fits_history_export_command_line(outimg,"fitrans",FITSH_FITRANS_VERSION,argc,argv);
+
+		trim_image(&img->i,mask,&outimg->i,outmask,ofx+cx*stepx,ofy+cy*stepy);
+
+		sprintf(filename,"%s-%.3d-%.3d.fits",cut_basename,cx,cy);
+		fw=fopenwrite(filename);
+		if ( fw==NULL )			
+		 {	fprint_error("unable to create output file '%s'",filename);
+			return(1);
+		 }
+		fits_backscale(outimg,outimg->i.read.bscale,outimg->i.read.bzero);
+	 	fits_write(fw,outimg); 
+		fclosewrite(fw);
+
+		fits_mask_free(outmask);
+		fits_free(outimg);
+	     }
 	 }
+
+	fits_mask_free(mask);
+	fits_free(img);
+
+	return(0);
   }
- else if ( is_shift )
-  {	trf->type=TRANS_POLYNOMIAL;
-	trf->order=1;
-	trf->nval=2;
-	trf->ox=trf->oy=0.0;
-	trf->scale=1.0;
- 	trf->vfits=(double **)tensor_alloc_2d(double,3,2);
-	trf->vfits[0][0]=shift_dx;
-	trf->vfits[0][1]=1.0;
-	trf->vfits[0][2]=0.0;
-	trf->vfits[1][0]=shift_dy;
-	trf->vfits[1][1]=0.0;
-	trf->vfits[1][2]=1.0;
-  }
+
+ /* image -> image, normal transformation: */
  else
-	trf=NULL;
+  {	char	*basename;
+	FILE	*fr,*fw;
+	fits	*img,*outimg;
+	char	**mask,**outmask;
 
- if ( trf != NULL && trf->nval != 2 )
-  {	fprint_error("input transformation is not an 2D -> 2D transformation");
-	return(1);
-  }
-
-
-/* Apply the required transformation: */
- if ( trf != NULL )
-	apply_transformation(&img->i,mask,&outimg->i,outmask,ofx,ofy,trf,int_method,is_invert);
- else if ( zratio>1 )
-	zoom_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,zratio);
- else if ( gratio>1 )
-	zoom_raw_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,gratio);
- else if ( sratio>1 )
-	shrink_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,sratio,do_median,do_avg_mode);
- else if ( bqscatt )
-	scatter_image(&img->i,mask,&outimg->i,outmask);
- else if ( sp.type || sp.filter )
-	combined_smooth_image(&img->i,mask,&outimg->i,outmask,&sp);
- else 
-	trim_image(&img->i,mask,&outimg->i,outmask,ofx,ofy);
-
- fits_mask_free(mask);
- if ( fdp.bitpix )
-  {	outimg->i.bit=fdp.bitpix;		}
- if ( fdp.is_scale )	
-  {	outimg->i.read.bscale=fdp.bscale,
-	outimg->i.read.bzero=fdp.bzero;
-  }
- else if ( outimg->i.bit<0 )
-  {	outimg->i.read.bscale=1.0,
-	outimg->i.read.bzero=0.0;
-  }
- fits_set_image_params(outimg);
- fits_backscale(outimg,outimg->i.read.bscale,outimg->i.read.bzero);
-
- mark_integerlimited_pixels(&outimg->i,outmask,outimg->i.bit,1,MASK_OVERSATURATED,MASK_OVERSATURATED);
- 
-/* Write the output image: */
- if ( is_flip_y )
-  {	double	*pd;
-	char	*pc;
-	int	i;
-	for ( i=0 ; i<nsy/2 ; i++ )
-	 {	pd=outimg->i.data[i],
-		outimg->i.data[i]=outimg->i.data[nsy-i-1],
-		outimg->i.data[nsy-i-1]=pd;
-		pc=outmask[i],
-		outmask[i]=outmask[nsy-i-1],
-		outmask[nsy-i-1]=pc;
+	basename=fits_basename(inimgfile,&frameno);
+	if ( (fr=fopenread(basename))==NULL )
+	 {	fprint_error("unable to open input file '%s'.",basename);
+		return(1);
 	 }
-  }
- if ( is_flip_x )
-  {	double	wd;
-	int	i,j,wc;
-	for ( i=0 ; i<nsy ; i++ )
-	 {	for ( j=0 ; j<nsx/2 ; j++ )
-		 {	wd=outimg->i.data[i][j],
-			outimg->i.data[i][j]=outimg->i.data[i][nsx-1-j];
-			outimg->i.data[i][nsx-1-j]=wd;
-			wc=outmask[i][j],
-			outmask[i][j]=outmask[i][nsx-1-j],
-			outmask[i][nsx-1-j]=wc;
+	img=fits_read_frame_to_image(fr,frameno);
+	fclose(fr);
+	if ( img==NULL )
+	 {	fprint_error("unable to interpret input as FITS data.");
+		return(1); 
+	 }
+	if ( img->i.dim != 2 && layer<0 )
+	 {	fprint_error("image dimension differs from 2 and no layer has been specified.");
+		return(1); 
+	 }
+	else if ( img->i.dim == 3 && layer>=0 && layer<img->i.naxis[2] )
+	 {	fitsimage	fi;
+		int		i,sx,sy;
+		double		***dcube;
+		sx=img->i.naxis[0];
+		sy=img->i.naxis[1];
+		fits_image_alloc(&fi,sx,sy);
+		fi.bit=img->i.bit;
+		fi.curr=img->i.curr;
+		fi.read=img->i.read;
+		dcube=(double ***)img->i.vdata;
+		for ( i=0 ; i<sy ; i++ )
+		 {	memcpy(fi.data[i],dcube[layer][i],sizeof(double)*sx);	}
+		fits_image_free(&img->i);
+		img->i=fi;
+		fits_image_set_params(&img->header,&img->i);
+	 }
+	fits_rescale(img);
+ 
+	logmsg(is_verbose,"[%d,%d]\n",img->i.sx,img->i.sy);
+
+	/* Create the mask for the valid pixels: */
+	mask=fits_mask_read_from_header(&img->header,img->i.sx,img->i.sy,NULL);
+	if ( inmasklist != NULL )
+	 {	if ( join_masks_from_files(mask,img->i.sx,img->i.sy,inmasklist) )
+		 {	fprint_error("unable to read one of the input mask files");
+			return(1);
 		 }
 	 }
+	fits_mask_mark_nans(&img->i,mask,MASK_NAN);
+
+	if ( nsx<0 )	is_flip_x=!is_flip_x,nsx=-nsx;
+	if ( nsy<0 )	is_flip_y=!is_flip_y,nsy=-nsy;
+
+	if ( nsx==0 && nsy==0 )
+	 {	if ( 1<=zratio_x || 1<=zratio_y )
+			nsx=img->i.sx*zratio_x,
+			nsy=img->i.sy*zratio_y;
+		else if ( 1<=gratio_x || 1<=gratio_y )
+			nsx=img->i.sx*gratio_x,
+			nsy=img->i.sy*gratio_y;
+		else if ( 1<=sratio_x || 1<=sratio_y )
+			nsx=(img->i.sx+sratio_x-1)/sratio_x,
+			nsy=(img->i.sy+sratio_y-1)/sratio_y;
+		else if ( 0<interleave_x && 0<interleave_y )
+			nsx=interleave_x,
+			nsy=interleave_y;
+		else if ( 0<repetitive_x && 0<repetitive_y )
+			nsx=repetitive_x,
+			nsy=repetitive_y;
+		else
+			nsx=img->i.sx,
+			nsy=img->i.sy;
+	 }
+
+	/* Create new FITS image for the output: */
+	outimg=fits_duplicate_empty(img);
+	if ( nsx != img->i.sx || nsy != img->i.sy )
+	 {	fits_free_image(outimg);
+		fits_alloc_image(outimg,nsx,nsy);
+		fits_reset_image(outimg);
+		fits_set_header_integer(outimg,"NAXIS1",FITS_SH_FIRST,nsx,NULL);
+		fits_set_header_integer(outimg,"NAXIS2",FITS_SH_FIRST,nsy,NULL);
+	 }
+	outmask=fits_mask_create_empty(nsx,nsy);
+
+	/* Read transformation file, if it has been specified in the command line: */
+	if ( transfile != NULL )
+	 {	FILE	*ft;
+		ft=fopenread(transfile);
+		if ( ft==NULL )	
+		 {	fprint_error("unable to open input transformation file '%s'",transfile);
+			return(1);
+		 }
+		i=transformation_read_data(ft,trf);
+		if ( i )
+		 {	fprint_error("unable to parse the contents of input transformation file '%s'",transfile);
+			return(1);
+		 }
+		fcloseread(ft);
+	 }
+	else if ( transparam != NULL )
+	 {	i=transformation_parse_params(transparam,trf);
+		if ( i )		
+		 {	fprint_error("unable to parse transformation string '%s'",transparam);
+			return(1);
+		 }
+	 }
+	else if ( is_shift )
+	 {	trf->type=TRANS_POLYNOMIAL;
+		trf->order=1;
+		trf->nval=2;
+		trf->ox=trf->oy=0.0;
+		trf->scale=1.0;
+	 	trf->vfits=(double **)tensor_alloc_2d(double,3,2);
+		trf->vfits[0][0]=shift_dx;
+		trf->vfits[0][1]=1.0;
+		trf->vfits[0][2]=0.0;
+		trf->vfits[1][0]=shift_dy;
+		trf->vfits[1][1]=0.0;
+		trf->vfits[1][2]=1.0;
+	 }
+	else
+		trf=NULL;
+
+	if ( trf != NULL && trf->nval != 2 )
+	 {	fprint_error("input transformation is not an 2D -> 2D transformation");
+		return(1);
+	 }
+
+	sqjacobi=1.0;
+
+	/* Apply the required transformation: */
+	if ( trf != NULL )
+	 {	apply_transformation(&img->i,mask,&outimg->i,outmask,ofx,ofy,trf,int_method,is_invert);
+		sqjacobi=apply_transformation_get_sqjacobi(&outimg->i,ofx,ofy,trf,is_invert);
+	 }	
+	else if ( 1<=zratio_x || 1<=zratio_y )
+	 {	zoom_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,zratio_x,zratio_y);
+		sqjacobi=sqrt(zratio_x*zratio_y);
+	 }
+	else if ( 1<=gratio_x || 1<=gratio_y )
+	 {	zoom_raw_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,gratio_x,gratio_y);
+		sqjacobi=sqrt(gratio_x*gratio_y);
+	 }
+	else if ( 1<=sratio_x || 1<=sratio_y )
+	 {	shrink_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,sratio_x,sratio_y,&sh_mode);
+		sqjacobi=1.0/(double)sqrt(sratio_x*sratio_y);
+	 }
+	else if ( 0<interleave_x && 0<interleave_y )
+	 {	interleave_image(&img->i,mask,&outimg->i,outmask,ofx,ofy,&sh_mode);
+		sqjacobi=0.0;
+	 }
+	else if ( 0<repetitive_x && 0<repetitive_y )
+	 {	repetitive_image(&img->i,mask,&outimg->i,outmask,ofx,ofy);
+		sqjacobi=0.0;
+	 }
+	else if ( bqscatt )
+		scatter_image(&img->i,mask,&outimg->i,outmask);
+	else if ( sp.type || sp.filter )
+		combined_smooth_image(&img->i,mask,&outimg->i,outmask,&sp);
+	else 
+		trim_image(&img->i,mask,&outimg->i,outmask,ofx,ofy);
+
+	fits_mask_free(mask);
+	if ( fdp.bitpix )
+	 {	outimg->i.bit=fdp.bitpix;		}
+	if ( fdp.is_scale )	
+	 {	outimg->i.read.bscale=fdp.bscale,
+		outimg->i.read.bzero=fdp.bzero;
+	 }
+	else if ( outimg->i.bit<0 )
+	 {	outimg->i.read.bscale=1.0,
+		outimg->i.read.bzero=0.0;
+	 }
+	fits_set_image_params(outimg);
+	fits_backscale(outimg,outimg->i.read.bscale,outimg->i.read.bzero);
+
+	mark_integerlimited_pixels(&outimg->i,outmask,outimg->i.bit,1,MASK_OVERSATURATED,MASK_OVERSATURATED);
+ 
+	/* Write the output image: */
+	if ( is_flip_y )
+	 {	double	wd;
+		int	i,j,wc;
+		for ( i=0 ; i<nsy/2 ; i++ )
+		 {	for ( j=0 ; j<nsx ; j++ )
+			 {	wd=outimg->i.data[i][j],
+				outimg->i.data[i][j]=outimg->i.data[nsy-1-i][j];
+				outimg->i.data[nsy-1-i][j]=wd;
+				wc=outmask[i][j],
+				outmask[i][j]=outmask[nsy-1-i][j],
+				outmask[nsy-1-i][j]=wc;
+			 }
+		 }
+	 }
+	if ( is_flip_x )
+	 {	double	wd;
+		int	i,j,wc;
+		for ( i=0 ; i<nsy ; i++ )
+		 {	for ( j=0 ; j<nsx/2 ; j++ )
+			 {	wd=outimg->i.data[i][j],
+				outimg->i.data[i][j]=outimg->i.data[i][nsx-1-j];
+				outimg->i.data[i][nsx-1-j]=wd;
+				wc=outmask[i][j],
+				outmask[i][j]=outmask[i][nsx-1-j],
+				outmask[i][nsx-1-j]=wc;
+			 }
+		 }
+	 }
+
+	fits_history_export_command_line(outimg,"fitrans",FITSH_FITRANS_VERSION,argc,argv);
+
+	fits_set_header_double(outimg,"SQJACOBI",FITS_SH_FIRST,sqjacobi,"square root of Jacobian determinant");
+
+	fits_mask_export_as_header(&outimg->header,1,outmask,nsx,nsy,NULL);
+	if ( outimgfile==NULL )	fw=stdout;
+	else			fw=fopenwrite(outimgfile);
+	if ( fw==NULL )		
+	 {	fprint_error("unable to create output image file '%s'",outimgfile);
+		return(1);
+	 }
+
+	fits_write(fw,outimg);
+	fclosewrite(fw);
+
+	return(0);
   }
-
- fits_history_export_command_line(outimg,"fitrans",FITSH_FITRANS_VERSION,argc,argv);
-
- fits_mask_export_as_header(&outimg->header,1,outmask,nsx,nsy,NULL);
- if ( outimgfile==NULL )	fw=stdout;
- else				fw=fopenwrite(outimgfile);
- if ( fw==NULL )		
-  {	fprint_error("unable to create output image file '%s'",outimgfile);
-	return(1);
-  }
-
- fits_write(fw,outimg);
- fclosewrite(fw);
-
- return(0);
 }
 
 /*****************************************************************************/
