@@ -6,7 +6,7 @@
 /*	- any dimensional coordinate sets (coordinate matching), and	     */
 /*	- identifiers (identifier matching).				     */
 /*****************************************************************************/
-#define	FITSH_GRMATCH_VERSION	"0.9d6"
+#define	FITSH_GRMATCH_VERSION	"0.9e0"
 /*****************************************************************************/
 
 #include <stdio.h>
@@ -29,6 +29,7 @@
 #include "math/tpoint.h"
 #include "math/trimatch.h"
 #include "math/cpmatch.h"
+#include "math/convexhull.h"
 
 #include "transform.h"
 #include "common.h"
@@ -93,6 +94,8 @@ typedef struct
 	int	ncolid;
 	
 	int	neg_ordering;
+
+	int	separator;
  } colinfo;
 
 typedef struct
@@ -139,6 +142,16 @@ int get_number_list(char *strcolid,int **rids,int *rnid)
  return(0);
 }
 
+int colinfo_reset(colinfo *col)
+{
+ col->colord=0,col->neg_ordering=0;
+ col->ncolid=col->ncolcoord=0;
+ col->colwgh=-1;
+ col->colids=col->colcoords=NULL;
+ col->separator=0;
+ return(0);
+}
+
 int normalize_columns(colinfo *col,char *strcolcoord,char *strcolid)
 {
  if ( col->colord>0 )		col->colord--,col->neg_ordering=0;
@@ -201,7 +214,10 @@ int read_match_data_points(FILE *fr,colinfo *col,iline **rils,int *rnil,int mtyp
 	ln++;
 	sbuff=strdup(rbuff);
 	remove_newlines_and_comments(rbuff);
-	cmd=tokenize_spaces_dyn(rbuff);
+
+	if ( col->separator )	cmd=tokenize_char_dyn(rbuff,col->separator);
+	else			cmd=tokenize_spaces_dyn(rbuff);
+
 	if ( cmd==NULL )	continue;
 	for ( n=0 ; cmd[n] != NULL ; )	n++;
 	if ( n==0 )		continue;
@@ -352,6 +368,8 @@ typedef struct
 
 	int	tri_level;	/* maximal triangulation level used	     */
 
+	double	hull_coverage;	/* coverage ratio of the convex hull points  */
+
  } matchpointstat;
 
 int match_compare_ordering(const void *vi1,const void *vi2)
@@ -558,6 +576,9 @@ int do_pointmatch(iline *refls,int nref,iline *inpls,int ninp,
 
  if ( mps != NULL && nhit>0 )
   {	double	ws,ns,wdd,ndd,nx,ny,dx,dy,dd;
+	point	*hullpoints;
+	int	nhull;
+	double	area_total,area_matched;
 
 	ws=wdd=0.0;
 	ns=ndd=0.0;
@@ -595,11 +616,43 @@ int do_pointmatch(iline *refls,int nref,iline *inpls,int ninp,
 	else		mps->nsigma=0.0;
 
 	mps->unitarity=calc_2d_unitarity(xfit,yfit,order);
+
+	hullpoints=(point *)malloc(sizeof(point)*ninp);
+
+	for ( i=0 ; i<ninp ; i++ )
+	 {	hullpoints[i].x=inpls[i].p.d.x;
+		hullpoints[i].y=inpls[i].p.d.y;
+	 }
+	nhull=convexhull_compute(hullpoints,ninp);
+	if ( 3<=nhull )	
+		area_total=convexhull_area(hullpoints,nhull);
+	else
+		area_total=0.0;
+
+	for ( i=0 ; i<nhit ; i++ )
+	 {	hullpoints[i].x=inpls[hits[i].idx[1]].p.d.x;
+		hullpoints[i].y=inpls[hits[i].idx[1]].p.d.y;
+	 }
+	nhull=convexhull_compute(hullpoints,nhit);
+	if ( 3<=nhull )	
+		area_matched=convexhull_area(hullpoints,nhull);
+	else
+		area_matched=0.0;
+
+	if ( 0<area_total )
+		mps->hull_coverage=area_matched/area_total;
+	else
+		mps->hull_coverage=0.0;
+
+	free(hullpoints);
+
+	
   }
  else if ( mps != NULL )
   {	mps->wsigma=0.0;
 	mps->nsigma=0.0;
 	mps->unitarity=-1.0;
+	mps->hull_coverage=0.0;
   }
 
  free(rftps);
@@ -973,6 +1026,7 @@ int fprint_grmatch_usage(FILE *fw)
 "Usage:\tgrmatch [-h|--help|--long-help|--wiki-help] [--version[-short]]\n"
 "\t[-C|--comment] [-V|--verbose]\n"
 "\t-r|--input-reference <reffile> [[-i|--input] <in>] [-o|--output <out>]\n"
+"\t[--separator-reference <char>|space] [--separator-input <char>|space]\n"
 "Point matching (default, --match-points can be omitted):\n"
 "\t--match-points [--col-ref <>,<>] [--col-inp <>,<>] \n"
 "\t[--col-ref-ordering [-]<> --col-inp-ordering [-]<>]\n"
@@ -989,6 +1043,8 @@ int fprint_grmatch_usage(FILE *fw)
  fprintf(fw,
 "Outputs:\n"
 "\t[--output|--output-matched|-o <line-pairs>] [--output-id <id-pairs>]\n"
+"\t[--output-matched-reference <matched-lines-from-reference>]\n"
+"\t[--output-matched-input <matched-lines-from-input>]\n"
 "\t[--output-excluded-{reference|input} <excluded-lines>]\n");
  fprintf(fw,
 "Fine-tuning of point matching:\n"
@@ -1020,11 +1076,18 @@ longhelp_entry grmatch_long_help[] =
 	"Comment the output (both the transformation file and the match file)." },
 
  { "Options for input/output specifications:", NULL },
- { "-r <referencefile>, --input-reference <referencefile>",
+ { "-r <file>, --reference <file> --input-reference <file>",
 	"Mandatory, name of the reference file." },
  { "<inputfile>, -i <inputile>, --input <inputfile>",
 	"Name of the input file. If this switch is omitted, the input is"
 	"read from stdin (specifying some input is mandatory)." },
+ { "--separator-reference <char>|space, --separator-input <char>|space",
+	"Character for separating the fields of the reference and the input "
+	"input files, respectively. By default, the separation is done using "
+	"whitespaces, it can be ephasized by defining 'space' here. "
+	"Otherwise, the character <char> should only be a single character. "
+	"For instance, use '--separator-reference ,' and/or "
+	"'--separator-input ,' to process CSV files. " },
  { "-o <output>, --output <output>, --output-matched <output>",
 	"Name of the output file, containing the matched lines. The "
 	"matched lines are pasted lines, the first part is from the "
@@ -1032,6 +1095,10 @@ longhelp_entry grmatch_long_help[] =
 	"two parts are concatenated by a TAB character. This switch is "
 	" optional, if it is not specified, no such output will "
 	"be generated. " },
+ { "--output-matched-reference <out>, --output-matched-input <out>",
+	"Name of the output file, containing the lines corresponding to "
+	"matches but only from the reference file or from "
+	"the input file, respectively. " },
  { "--output-excluded-reference <out>, --output-excluded-input <out>",
 	"Names of the files which contain the valid but excluded lines "
 	"from the reference and from the input. These outputs are "
@@ -1336,6 +1403,7 @@ int fprint_pointmatch_stat(FILE *fw,int nhit,int nref,int ninp,matchpointstat *m
  fprintf(fw,"# Timing: %.3f %.3f %.3f %d (total, triangle match, coordinate match: in seconds; iterations)\n",
 		mps->time_total,mps->time_trimatch,mps->time_symmatch,mps->nmiter);
  fprintf(fw,"# Triangulation: %d (maximal triangulation level)\n",mps->tri_level);
+ fprintf(fw,"# Convex hull coverage ratio: %g\n",mps->hull_coverage);
  fprintf(fw,"# All: %g %g %g %d %d %d %.2f %.3f %.3f %.3f %d %d\n",
 	mps->nsigma,mps->wsigma,mps->unitarity,
 	nhit,nref,ninp,rat*100.0,
@@ -1357,7 +1425,7 @@ int fprint_general_stat(FILE *fw,int nhit,int nref,int ninp)
 }
 
 
-int fwrite_excluded_lines(FILE *fw,iline *refls,int nref,cphit *hits,int nhit,int t)
+int fwrite_matched_or_excluded_lines(FILE *fw,iline *refls,int nref,cphit *hits,int nhit,int type,int is_excluded)
 {
  char	*excflag;
  int	i,l;
@@ -1365,14 +1433,18 @@ int fwrite_excluded_lines(FILE *fw,iline *refls,int nref,cphit *hits,int nhit,in
  excflag=(char *)malloc(nref);
  memset(excflag,0,nref);
 
- if ( t )	t=1;
+ if ( type )	type=1;
 
  for ( i=0 ; i<nhit && hits != NULL ; i++ )
-  {	l=hits[i].idx[t];
+  {	l=hits[i].idx[type];
 	excflag[l]=1;
   }
+ if ( is_excluded )
+  {	for ( i=0 ; i<nref ; i++ )
+	 {	excflag[i]=(!excflag[i]);	}
+  }
  for ( i=0 ; i<nref ; i++ )
-  {	if ( excflag[i] )	continue;
+  {	if ( !excflag[i] )	continue;
 	fprintf(fw,"%s\n",refls[i].line);
   }
 
@@ -1386,9 +1458,10 @@ int main(int argc,char *argv[])
  FILE		*fr,*fw;
  int		i,order,hintorder,nvar,is_help;
  char		*infile,*outfile,*reffile,*outlinefile,*outidfile,
+		*outmreffile,*outminpfile,
 		*outxreffile,*outxinpfile,
-		*inpcolids,*inpcolcoords,
-		*refcolids,*refcolcoords,
+		*inpcolids,*inpcolcoords,*inpseparator,
+		*refcolids,*refcolcoords,*refseparator,
 		*outtransfile,*hntransfile,*intransfile,*intransparam,
 		*triparam,*fitparam,*wghparam;
  colinfo 	colref,colinp;
@@ -1412,6 +1485,7 @@ int main(int argc,char *argv[])
  else				progbasename=argv[0];
 
  infile=outfile=reffile=NULL;
+ outmreffile=outminpfile=NULL;
  outxreffile=outxinpfile=NULL;
  outidfile=outlinefile=NULL;
  intransfile=outtransfile=intransparam=triparam=fitparam=wghparam=NULL;
@@ -1420,14 +1494,14 @@ int main(int argc,char *argv[])
 
  hintorder=-1;
  hntransfile=NULL;
- 
- colref.colord=0,colref.neg_ordering=0;
- colref.ncolid=colref.ncolcoord=0;refcolids=refcolcoords=NULL;colref.colwgh=-1;
- colref.colids=colref.colcoords=NULL;
- colinp.colord=0,colinp.neg_ordering=0;
- colinp.ncolid=colinp.ncolcoord=0;inpcolids=inpcolcoords=NULL;colinp.colwgh=-1;
- colref.colids=colinp.colcoords=NULL;
 
+ refcolids=refcolcoords=NULL;
+ inpcolids=inpcolcoords=NULL;
+ refseparator=NULL;
+ inpseparator=NULL;
+ colinfo_reset(&colref);
+ colinfo_reset(&colinp);
+ 
  matchtype=MATCH_POINTS; matchread=0;
  ambig=AMBIG_NONE;
 
@@ -1454,9 +1528,13 @@ int main(int argc,char *argv[])
 	"--mediawiki-help|--help-mediawiki|--wiki-help|--help-wiki:%SN3f%q",&is_help,
 	"-i|--input:%s",&infile,
 	"-r|--reference|--input-reference:%s",&reffile,
+	"--separator-reference:%s",&refseparator,
+	"--separator-input:%s",&inpseparator,
 	"-o|--output|-om|--output-match|--output-matched:%s",&outfile,
 	"--output-id|--output-ids:%s",&outidfile,
 	"-L|--output-line|--output-lines:%s",&outlinefile,
+	"--output-matched-reference:%s",&outmreffile,
+	"--output-matched-input:%s",&outminpfile,
 	"--output-excluded-reference:%s",&outxreffile,
 	"--output-excluded-input:%s",&outxinpfile,
 
@@ -1537,6 +1615,22 @@ int main(int argc,char *argv[])
   {	fprint_error("invalid column specification");
 	return(1);
   }
+ if ( refseparator != NULL && strlen(refseparator)==1 )
+	colref.separator=refseparator[0];
+ else if ( refseparator != NULL && strcmp(refseparator,"space")==0 )
+	colref.separator=0;
+ else if ( refseparator != NULL )
+  {	fprint_error("invalid reference separator specification: '%s'",refseparator);
+	return(1);
+  }
+ if ( inpseparator != NULL && strlen(inpseparator)==1 )
+	colinp.separator=inpseparator[0];
+ else if ( inpseparator != NULL && strcmp(inpseparator,"space")==0 )
+	colinp.separator=0;
+ else if ( inpseparator != NULL )
+  {	fprint_error("invalid input separator specification: '%s'",inpseparator);
+	return(1);
+  }
 
  if ( (matchread & MATCH_READ_COORDS) && colref.ncolcoord != colinp.ncolcoord )
   {	fprint_error("reference and input dimension mismatched");
@@ -1797,13 +1891,32 @@ int main(int argc,char *argv[])
 	fclosewrite(fw);
   }
 
+ if ( outmreffile != NULL )
+  {	fw=fopenwrite(outmreffile);
+	if ( fw==NULL )	
+	 {	fprint_error("unable to create output file '%s' for matched reference lines",outmreffile);
+		return(1);
+	 }
+	fwrite_matched_or_excluded_lines(fw,refls,nref,hits,nhit,0,0);
+	fclosewrite(fw);
+  }
+ if ( outminpfile != NULL )
+  {	fw=fopenwrite(outminpfile);
+	if ( fw==NULL )	
+	 {	fprint_error("unable to create output file '%s' for matched input lines",outminpfile);
+		return(1);
+	 }
+	fwrite_matched_or_excluded_lines(fw,inpls,ninp,hits,nhit,1,0);
+	fclosewrite(fw);
+  }
+	
  if ( outxreffile != NULL )
   {	fw=fopenwrite(outxreffile);
 	if ( fw==NULL )	
 	 {	fprint_error("unable to create output file '%s' for excluded reference lines",outxreffile);
 		return(1);
 	 }
-	fwrite_excluded_lines(fw,refls,nref,hits,nhit,0);
+	fwrite_matched_or_excluded_lines(fw,refls,nref,hits,nhit,0,1);
 	fclosewrite(fw);
   }
  if ( outxinpfile != NULL )
@@ -1812,7 +1925,7 @@ int main(int argc,char *argv[])
 	 {	fprint_error("unable to create output file '%s' for excluded input lines",outxinpfile);
 		return(1);
 	 }
-	fwrite_excluded_lines(fw,inpls,ninp,hits,nhit,1);
+	fwrite_matched_or_excluded_lines(fw,inpls,ninp,hits,nhit,1,1);
 	fclosewrite(fw);
   }
 	
