@@ -1,3 +1,4 @@
+/*****************************************************************************/
 /* grtrans.c								     */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Command line utility to fit general two-dimensional transformations.	     */
@@ -21,7 +22,7 @@
 #include "math/fit/lmfit.h"
 #include "math/poly.h"
 #include "math/polyfit.h"
-#include "wcs.h"
+#include "projection.h"
 
 #include "transform.h"
 #include "common.h"
@@ -30,6 +31,26 @@
 #define __extension__ 
 #endif 
 
+typedef struct
+ {      double                  ra0,de0,roll0;
+	double			qr0,qi0,qj0,qk0;
+	double			max_distance;
+        int                     type,order;
+        matrix                  mproj;
+        double                  zfactor;
+        projection_distort      distort;
+ } projection_data;
+
+typedef struct
+ {	projection_data init;
+	double  *pixpoly[2];
+	double  *prjpoly[2];
+	double  crpix1,crpix2;
+	double  cd11,cd12,cd21,cd22;
+	double  res_prj,res_pix;
+ } wcs_data;
+
+
 /*****************************************************************************/
 
 int	is_verbose,is_comment;
@@ -78,19 +99,21 @@ typedef struct
  {	double	x,y;
 	double	weight,delta;
 	double	*vals;
- } fpoint;
+ } fitpoint;
 
 /*****************************************************************************/
 
-int fit_wcs(fpoint *fps,int nfp,wcsinit *wi,wcsdata *wcs)
+int grtrans_wcs_perform_fit(fitpoint *fps,int nfp,projection_data *wi,wcs_data *wcs)
 {
- int	i,k,nvar,order;
- point	*ps;
- double	*vfits[4],cdelt1,cdelt2,idelt,lxx,lxy,lyx,lyy;
+ int	i,k,m,nvar,order;
+ point  *p_pix;
+ point  *p_prj;
+ point  *ps;
+ double	*vfits[4],cd11,cd12,cd21,cd22,crpix1,crpix2,det,sig2;
  double	ra,de,x,y,z;
 
  if ( wi==NULL || wcs==NULL )	return(-1);
- memmove(&wcs->init,wi,sizeof(wcsinit));
+ memmove(&wcs->init,wi,sizeof(projection_data));
  
  order=wcs->init.order;
  if ( order<=0 )		return(-1);
@@ -100,108 +123,217 @@ int fit_wcs(fpoint *fps,int nfp,wcsinit *wi,wcsdata *wcs)
  for ( k=0 ; k<4 ; k++ )
   {	vfits[k]=(double *)malloc(sizeof(double)*nvar);		}
 
- ps=(point *)malloc(sizeof(point)*nfp);
+ p_pix=(point *)malloc(sizeof(point)*nfp);
+ p_prj=(point *)malloc(sizeof(point)*nfp);
+ ps   =(point *)malloc(sizeof(point)*nfp);
 
+ /* WCS fit stage 1: prepare projection: */
  for ( i=0 ; i<nfp ; i++ )
   {	ra=fps[i].x,
 	de=fps[i].y;
-	wcs_get_projected_coords(ra,de,wcs->init.ra0,wcs->init.de0,&x,&y,&z);
-	wcs_project_distort(wcs->init.type,&x,&y,&z);
-	fps[i].x=x*M_R2D;
-	fps[i].y=y*M_R2D;
+	/* Here the roll angle is zero by definition, it is handled differently: */
+	projection_do_coord(ra,de,wcs->init.ra0,wcs->init.de0,0.0,&x,&y,&z);
+	projection_do_distortion(wcs->init.type,NULL,&x,&y,&z);
+        p_prj[i].x=x*M_R2D;
+        p_prj[i].y=y*M_R2D;
+        p_pix[i].x=fps[i].vals[0];
+        p_pix[i].y=fps[i].vals[1];
   }
 
+ /* WCS fit stage 2: get an initial hint for CRPIX[12] and CD[12]_[12]: */
  for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].x=fps[i].x,
-	ps[i].y=fps[i].y;
+  {	ps[i].x=p_pix[i].x;
+	ps[i].y=p_pix[i].y;
 	ps[i].weight=1.0;
-	ps[i].value=fps[i].vals[0];
   }
+ for ( i=0 ; i<nfp ; i++ )
+  {	ps[i].value=p_prj[i].x;		}
  fit_2d_poly(ps,nfp,1,vfits[0],0,0,1);
  for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].value=fps[i].vals[1];		}
+  {	ps[i].value=p_prj[i].y;		}
  fit_2d_poly(ps,nfp,1,vfits[1],0,0,1);
- lxx=vfits[0][1],lxy=vfits[0][2];
- lyx=vfits[1][1],lyy=vfits[1][2];
- idelt=sqrt(0.5*(lxx*lxx+lxy*lxy+lyx*lyx+lyy*lyy));
- if ( lxx*lyy>0.0 )		/* Det >  0	*/
-  {	if ( lxx+lyy>=0.0 )	/* Tr  >= 0	*/
-	 {	cdelt1=+1.0/idelt;
-		cdelt2=+1.0/idelt;
-	 }
-	else
-	 {	cdelt1=-1.0/idelt;
-		cdelt2=-1.0/idelt;
-	 }
-  }
- else
-  {	if ( lxx>0.0 )
-	 {	cdelt1=+1.0/idelt;
-		cdelt2=-1.0/idelt;		
-	 }
-	else
-	 {	cdelt1=-1.0/idelt;
-		cdelt2=+1.0/idelt;
-	 }
-  }
+ cd11=vfits[0][1],cd12=vfits[0][2];
+ cd21=vfits[1][1],cd22=vfits[1][2];
+ det=(cd11*cd22-cd12*cd21);
+ crpix1=-(+cd22*vfits[0][0]-cd12*vfits[1][0])/det;
+ crpix2=-(-cd21*vfits[0][0]+cd11*vfits[1][0])/det;
 
+ /* WCS fit stage 3: refine parameters by considering higher order distortions: */
+ if ( 2 <= order )
+  {	/* re-iterate CRPIX1 and CRPIX2: */
+	for ( k=0 ; k<3 ; k++ )
+	 {	for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].x=p_pix[i].x-crpix1;
+			ps[i].y=p_pix[i].y-crpix2;
+			ps[i].weight=1.0;
+		 }
+		for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].value=p_prj[i].x;		}
+		fit_2d_poly(ps,nfp,order,vfits[0],0,0,1);
+		for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].value=p_prj[i].y;		}
+		fit_2d_poly(ps,nfp,order,vfits[1],0,0,1);
+		cd11=vfits[0][1],cd12=vfits[0][2];
+		cd21=vfits[1][1],cd22=vfits[1][2];
+		det=(cd11*cd22-cd12*cd21);
+		crpix1 += -(+cd22*vfits[0][0]-cd12*vfits[1][0])/det;
+		crpix2 += -(-cd21*vfits[0][0]+cd11*vfits[1][0])/det;
+	  }
+	
+	/* re-iterate CD1_1, CD1_2, CD2_1 and CD2_2: */
+	 for ( k=0 ; k<1 ; k++ )
+	  {	double	det,cx11,cx12,cx21,cx22;
+		det=(cd11*cd22-cd12*cd21);
+		for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].x=p_pix[i].x-crpix1;
+			ps[i].y=p_pix[i].y-crpix2;
+			ps[i].weight=1.0;
+		 }
+		for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].value=(+cd22*p_prj[i].x-cd12*p_prj[i].y)/det;	}
+		fit_2d_poly(ps,nfp,order,vfits[0],0,0,1);
+		for ( i=0 ; i<nfp ; i++ )
+		 {	ps[i].value=(-cd21*p_prj[i].x+cd11*p_prj[i].y)/det;	}
+		fit_2d_poly(ps,nfp,order,vfits[1],0,0,1);
+		
+		cx11=cd11*vfits[0][1]+cd12*vfits[1][1];
+		cx12=cd11*vfits[0][2]+cd12*vfits[1][2];
+		cx21=cd21*vfits[0][1]+cd22*vfits[1][1];
+		cx22=cd21*vfits[0][2]+cd22*vfits[1][2];
+		cd11=cx11;
+		cd12=cx12;
+		cd21=cx21;
+		cd22=cx22;
+	  }
+   }
+
+ /* force these polynomial terms to be zero and unity, according to the standard: */
+ vfits[0][0]=0.0;
+ vfits[0][1]=1.0;
+ vfits[0][2]=0.0;
+ vfits[1][0]=0.0;
+ vfits[1][1]=0.0;
+ vfits[1][2]=1.0;
+
+ sig2=0.0;
  for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].x=fps[i].x/cdelt1,
-	ps[i].y=fps[i].y/cdelt2;
-	ps[i].value=fps[i].vals[0]-ps[i].x;
+  {	double	x,y,px,py;
+ 	px=p_pix[i].x-crpix1;
+	py=p_pix[i].y-crpix2;
+	x=eval_2d_poly(px,py,order,vfits[0],0,0,1);
+	y=eval_2d_poly(px,py,order,vfits[1],0,0,1);
+	px=cd11*x+cd12*y;
+	py=cd21*x+cd22*y;
+	px-=p_prj[i].x;
+	py-=p_prj[i].y;
+	sig2+=px*px+py*py;
+  };
+ if ( 0<nfp )	sig2=sqrt(sig2/(double)nfp);
+ wcs->res_prj=sig2;
+
+ /* derive the CD^(-1)*(\xi,\eta) => (x-CRPIX1,y-CRPIX2) coefficients */
+ det=(cd11*cd22-cd12*cd21);
+ for ( i=0 ; i<nfp ; i++ )
+  {	ps[i].x=(+cd22*p_prj[i].x-cd12*p_prj[i].y)/det;
+	ps[i].y=(-cd21*p_prj[i].x+cd11*p_prj[i].y)/det;	
+	ps[i].weight=1.0;
   }
+ for ( i=0 ; i<nfp ; i++ )
+  {	ps[i].value=p_pix[i].x-crpix1;		}
  fit_2d_poly(ps,nfp,order,vfits[2],0,0,1);
  for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].value=fps[i].vals[1]-ps[i].y;		}
+  {	ps[i].value=p_pix[i].y-crpix2;		}
  fit_2d_poly(ps,nfp,order,vfits[3],0,0,1);
+ vfits[2][0]=0.0;
+ vfits[3][0]=0.0;
 
+ /* compute residual of the CD^(-1)*(\xi,\eta) => (x-CRPIX1,y-CRPIX2) fit: */
+ det=(cd11*cd22-cd12*cd21);
+ sig2=0.0;
  for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].x=fps[i].vals[0]-vfits[2][0];
-	ps[i].y=fps[i].vals[1]-vfits[3][0];
- 	ps[i].value=(fps[i].x/cdelt1-ps[i].x);	
+  {	double	x,y,px,py;
+	px=(+cd22*p_prj[i].x-cd12*p_prj[i].y)/det;
+	py=(-cd21*p_prj[i].x+cd11*p_prj[i].y)/det;	
+	x=eval_2d_poly(px,py,order,vfits[2],0,0,1);
+	y=eval_2d_poly(px,py,order,vfits[3],0,0,1);
+	px=x-(p_pix[i].x-crpix1);
+	py=y-(p_pix[i].y-crpix2);
+	sig2+=px*px+py*py;
+  };
+ if ( 0<nfp )	sig2=sqrt(sig2/(double)nfp);
+ wcs->res_pix=sig2;
+
+ /* save coefficients: */
+ wcs->crpix1=crpix1;
+ wcs->crpix2=crpix2;
+ wcs->cd11=cd11;
+ wcs->cd12=cd12;
+ wcs->cd21=cd21;
+ wcs->cd22=cd22;
+
+ /* re-scale polynomial terms: divide A_ij, B_ij, AP_ij and BP_ij by (i!j!): */
+ k=0;
+ m=1;
+ for ( i=0 ; i<=order ; i++ )
+  {	int	j,w;
+	w=m;
+	for ( j=0 ; j<=i ; j++ )
+	 {	vfits[0][k] /= (double)w;
+		vfits[1][k] /= (double)w;
+		vfits[2][k] /= (double)w;
+		vfits[3][k] /= (double)w;
+		if ( j<i )	w=(w/(i-j))*(j+1);
+		k++;
+	 }
+	m=m*(i+1);
   }
- fit_2d_poly(ps,nfp,order,vfits[0],0,0,1);
- for ( i=0 ; i<nfp ; i++ )
-  {	ps[i].value=(fps[i].y/cdelt2-ps[i].y);		}
- fit_2d_poly(ps,nfp,order,vfits[1],0,0,1);
 
- wcs->crpix1=vfits[2][0];
- wcs->crpix2=vfits[3][0];
- wcs->cdelt1=cdelt1;
- wcs->cdelt2=cdelt2;
+ /* subtract unity matrix from the linear part of the reverse transformation: */
+ vfits[2][1]-=1.0;
+ vfits[3][2]-=1.0;
 
+ /* save polynomial terms: */ 
  wcs->pixpoly[0]=vfits[0];
  wcs->pixpoly[1]=vfits[1];
  wcs->prjpoly[0]=vfits[2];
  wcs->prjpoly[1]=vfits[3];
+
+ free(ps);
+ free(p_prj);
+ free(p_pix);
  
  return(0);
 }
 
-int dump_wcs_coeff_headers(FILE *fw,char *type,int order,double *coeff)
+static int grtrans_wcs_dump_coeff_headers(FILE *fw,char *type,int o0,int order,double *coeff)
 {
  int	i,j,k;
+
  fprintf(fw,"%s_ORDER=%d,\n",type,order);
- for ( i=1,k=1 ; i<=order ; i++ )
+
+ for ( i=0,k=0 ; i<o0 ; i++ )
+	k+=(i+1);
+
+ for ( i=o0 ; i<=order ; i++ )
   {	for ( j=0 ; j<=i ; j++,k++ )
 	 {	fprintf(fw,"%s_%d_%d=%.12g,\n",type,i-j,j,coeff[k]);	}
   }
  return(0);
 }
-int dump_wcs_headers(FILE *fw,wcsdata *wcs)
+int grtrans_wcs_dump_headers(FILE *fw,wcs_data *wcs)
 {
  char	*ttype;
 
  fprintf(fw,"WCSAXES=%d,\n",2);
 
  switch ( wcs->init.type )
-  {  case WCS_SIN:
+  {  case PROJECTION_SIN:
 	ttype="SIN";
 	break;
-     case WCS_TAN:
+     case PROJECTION_TAN:
 	ttype="TAN";
 	break;
-     case WCS_ARC:
+     case PROJECTION_ARC:
 	ttype="ARC";
 	break;
      default:
@@ -209,26 +341,38 @@ int dump_wcs_headers(FILE *fw,wcsdata *wcs)
 	break;
   }
 
- fprintf(fw,"CTYPE1='RA---%s-SIP',\n",ttype);
- fprintf(fw,"CTYPE2='DEC--%s-SIP',\n",ttype);
+ if ( 2<=wcs->init.order )
+  {	fprintf(fw,"CTYPE1='RA---%s-SIP',\n",ttype);
+	fprintf(fw,"CTYPE2='DEC--%s-SIP',\n",ttype);
+  }
+ else
+  {	fprintf(fw,"CTYPE1='RA---%s',\n",ttype);
+	fprintf(fw,"CTYPE2='DEC--%s',\n",ttype);
+  }
+
  fprintf(fw,"CRVAL1=%.4f,\n",wcs->init.ra0);
  fprintf(fw,"CRVAL2=%.4f,\n",wcs->init.de0);
- fprintf(fw,"CDELT1=%.12g,\n",wcs->cdelt1);
- fprintf(fw,"CDELT2=%.12g,\n",wcs->cdelt2);
- fprintf(fw,"CRPIX1=%.3f,\n",wcs->crpix1+1.0);
- fprintf(fw,"CRPIX2=%.3f,\n",wcs->crpix2+1.0);
+ fprintf(fw,"CD1_1=%.12g,\n",wcs->cd11);
+ fprintf(fw,"CD1_2=%.12g,\n",wcs->cd12);
+ fprintf(fw,"CD2_1=%.12g,\n",wcs->cd21);
+ fprintf(fw,"CD2_2=%.12g,\n",wcs->cd22);
+ fprintf(fw,"CRPIX1=%.3f,\n",wcs->crpix1+0.5);
+ fprintf(fw,"CRPIX2=%.3f,\n",wcs->crpix2+0.5);
 
- dump_wcs_coeff_headers(fw,"A" ,wcs->init.order,wcs->pixpoly[0]);
- dump_wcs_coeff_headers(fw,"B" ,wcs->init.order,wcs->pixpoly[1]);
- dump_wcs_coeff_headers(fw,"AP",wcs->init.order,wcs->prjpoly[0]);
- dump_wcs_coeff_headers(fw,"BP",wcs->init.order,wcs->prjpoly[1]);
+ if ( 2<=wcs->init.order )
+  {	grtrans_wcs_dump_coeff_headers(fw,"A" ,2,wcs->init.order,wcs->pixpoly[0]);
+	grtrans_wcs_dump_coeff_headers(fw,"B" ,2,wcs->init.order,wcs->pixpoly[1]);
+	grtrans_wcs_dump_coeff_headers(fw,"AP",1,wcs->init.order,wcs->prjpoly[0]);
+	grtrans_wcs_dump_coeff_headers(fw,"BP",1,wcs->init.order,wcs->prjpoly[1]);
+  }
 
- fprintf(fw,"CROTA2=0.0\n");
+ fprintf(fw,"WCSR_PRJ=%.5g,\n",wcs->res_prj);
+ fprintf(fw,"WCSR_PIX=%.5g\n",wcs->res_pix);
 
  return(0);
 }
 
-int free_wcs(wcsdata *wcs)
+int grtrans_wcs_data_free(wcs_data *wcs)
 {
  if ( wcs->pixpoly[0] != NULL )	free(wcs->pixpoly[0]);
  if ( wcs->pixpoly[1] != NULL )	free(wcs->pixpoly[1]);
@@ -241,12 +385,12 @@ int free_wcs(wcsdata *wcs)
 
 static char *wrninappr="Warning: inappropriate content in line %d, skipped.\n";
 
-int read_data_points(FILE *fr,colinfo *col,fpoint **rfps,int *rnfp)
+static int grtrans_read_data_points(FILE *fr,colinfo *col,fitpoint **rfps,int *rnfp)
 {
- char	*rbuff,**cmd;
- fpoint *fps;
- int	nfp,n,k,i,j,ln;
- double	xr,yr,wr,w,*vals;
+ char		*rbuff,**cmd;
+ fitpoint	*fps;
+ int		nfp,n,k,i,j,ln;
+ double		xr,yr,wr,w,*vals;
 
  fps=NULL;
  nfp=0;
@@ -288,8 +432,8 @@ int read_data_points(FILE *fr,colinfo *col,fpoint **rfps,int *rnfp)
 	 }
 	else	wr=1.0;
 
-	if ( nfp==0 )	fps=(fpoint *)malloc(sizeof(fpoint));
-	else		fps=(fpoint *)realloc(fps,sizeof(fpoint)*(nfp+1));
+	if ( nfp==0 )	fps=(fitpoint *)malloc(sizeof(fitpoint));
+	else		fps=(fitpoint *)realloc(fps,sizeof(fitpoint)*(nfp+1));
 	vals=(double *)malloc(sizeof(double)*col->nval);
 
 	k=0;
@@ -321,7 +465,7 @@ int read_data_points(FILE *fr,colinfo *col,fpoint **rfps,int *rnfp)
 
 /*****************************************************************************/
 
-int parse_number_list(char *p,int *rn,int **rfitc)
+static int grtrans_parse_number_list(char *p,int *rn,int **rfitc)
 {
  int	w,n,*fitc;
 
@@ -367,19 +511,17 @@ int fprint_grtrans_usage(FILE *fw)
 "\t[--col-weight <>] [--col-sdk <>,<>,<> [--col-out-sdk <>,<>,<>]]\n"
 "\t[--preserve-comments]\n");
  fprintf(fw,
-"WCS conversion and fitting parameters:\n"
-"\t[--wcs [sin|arc|tan],order=<distort.poly.order>,ra=<RA>,dec=<DEC>,\n"
-"\t       [degrees|radians|scale=<scale>]] [--col-radec|--col-pixel <>,<>]\n");
+"Spherical projections:\n"
+"\t[--proj [sin|arc|tan],ra=<RA>,dec=<DEC>,[roll=<ROLL>],[max=<max>],\n"
+"\t        [degrees|radians|scale=<scale>],\n"
+"\t        [k1=<K1>,k2=<K2>,...,p1=<P1>,p2=<P2>,...]\n"
+"\t[--proj [...],qr=<quat_r>,qi=<quat_i>,qj=<quat_j>,qk=<quat_k>]\n"
+"\t[--col-radec <>,<>|--col-pixel <>,<>]\n");
  fprintf(fw,
-"Note that 'WCS fitting' and 'WCS conversion' are three different things.\n"
-"Because of 'fitting WCS' is a derivation of some FITS headers required by the\n"
-"standard (see e.g. Calabretta & Greisen, A&A, 395, 1077), this invocation fits\n"
-"the distortion coefficients and dumps the appropriate keyword=value pairs.\n");
- fprintf(fw,
-"'WCS conversion' is something different: with this, one can project RA/DEC\n"
-"values (using a projection method: orthographic, arc or gnomonic) and scale\n"
-"into plane and vice versa. The confusion is due to the fact that both things\n"
-"require almost the same external parameters, given after the switch '--wcs'.\n");
+"Parameters related to fitting WCS-related FITS keywords:\n"
+"\t[--wcs  [sin|arc|tan],order=<distort.poly.order>,ra=<RA>,dec=<DEC>\n"
+"\t --col-radec <>,<> --col-pixel <>,<>]\n");
+
  return(0);
 }
 
@@ -508,7 +650,7 @@ longhelp_entry grtrans_long_help[]=
 	"result of the composition." },
 
  { "Options for spherical projection and deprojection:", NULL },
- { "--wcs [sin|arc|tan],ra=<R>,dec=<D>,[degrees|radians|scale=<S>",
+ { "--proj, --project [sin|arc|tan],ra=<R>,dec=<D>,[roll=<F>],[max=<max_distance>],[qr=<R>,qi=<I>,qj=<J>,qk=<K>],[degrees|radians|scale=<S>",
 	__extension__
 	"This set of directives specify  the  common  parameters  of  "
 	"the spherical  projection  or  deprojection.  The  \"sin\",  \"arc\" "
@@ -516,11 +658,22 @@ longhelp_entry grtrans_long_help[]=
 	"arc  and gnomonic,  respectively.  The values after \"ra\" and "
 	"\"dec\" (<R> and <D>) specify the center of the  projection  "
 	"(right  ascension  and declination, respectively, in degrees). "
+	"The optional parameter <F> is for the roll angle (in degrees). "
 	"The \"degrees\", \"radians\" or the \"scale=<S>\" directives "
 	"specify the scaling of  the  output. The directive \"degrees\" "
 	"is equivalent to set \"scale=57.29577951308232087721\" "
 	"(180  over \\Pi),  this  is  the default. The directive "
-	"\"radians\" is equivalent to set \"scale=1\"." },
+	"\"radians\" is equivalent to set \"scale=1\". " 
+	"Alternatively to the right  ascension, declination and roll "
+	"parameterization, one can use quaternion-based pointing here "
+	"by specifying qr, qi, qj, qk for the real, and the three "
+	"imaginary components, respectively." 
+	"By default, this projection tries to compute the projected coordinates "
+	"for all of the positions presented in the input list. However, it "
+	"might yield unexpected results in certain cases (e.g. an input list "
+	"on a nearly full sphere and orthographic projection). One can use the "
+	"\"max=<max_distance>\" parameter to filter objects "
+	"beyond this maximum distance. " },
  { "--col-radec <r>,<d>", 
 	"Column indices for RA and DEC values. This option implies "
 	"projection." },
@@ -541,9 +694,11 @@ longhelp_entry grtrans_long_help[]=
 	"seem to be  more than enough).  The center of the fit is set "
 	"by \"ra\" and \"dec\" (RA and DEC, in degrees). The distortion "
 	"order  is  specified by  order.  Note  that  the RA and DEC values "
-	"specified here can only be an assumed values. " },
+	"specified here can only be an assumed values, however, the larger "
+	"the optical distortions are, the better to have RA and DEC values "
+	"close to the optical axis. " },
  { "--col-ref <>,<>",
-	"Column indices for the Ra and Dec values." },
+	"Column indices for the RA and DEC values." },
  { "--col-fit <>,<>",
 	"Column indices for the pixel values." },
  { "Note that in this case, the set of the appropriate  FITS  keyword=value "
@@ -581,7 +736,7 @@ longhelp_entry grtrans_long_help[]=
 	"The  first  step  is to make a projection from the sky, "
 	"centered around the center of our image:", NULL }, { "", NULL },
 
- { "", 	"grtrans --input catalog.dat --wcs tan,degrees,ra=220,dec=25 "
+ { "", 	"grtrans --input catalog.dat --proj tan,degrees,ra=220,dec=25 "
 	"--col-radec 2,3 --col-out 5,6 --output img.proj"  },
 
  { "The second step is the point matching:", NULL}, {"", NULL },
@@ -665,18 +820,21 @@ int fprint_grtrans_long_help(FILE *fw,int is_wiki)
 
 /*****************************************************************************/
 
+#define		PROJ_RADEC_TO_PIXEL	
+#define		PROJ_PIXEL_TO_RADEC
+
 int main(int argc,char *argv[])
 {
  FILE		*fr,*fw;
  int		i,k,order,nvar,is_help,is_invert,is_ieee;
  char		*infile,*outfile,*outtransfile,*intransfile,
 		*colfitstr,*coloutstr,*colsdkstr,*colsdkoutstr,
-		*wcsparam,*weightparam;
- wcsdata	wcs_data,*wcs;
- int		wcsconvert;
+		*projection_param,*weightparam;
+ wcs_data	wcs_data_static,*wcs;
+ int		projection_direction;
  colinfo 	col;
  int		is_preserve_comments;
- fpoint		*fps;
+ fitpoint	*fps;
  point		*ps;
  int		nfp,niter,nt,padsize;
  double		rejlevel;
@@ -691,7 +849,7 @@ int main(int argc,char *argv[])
  is_comment=is_verbose=is_help=is_ieee=0;
  colfitstr=coloutstr=colsdkstr=colsdkoutstr=NULL;
 
- wcsparam=NULL,wcsconvert=0;
+ projection_param=NULL,projection_direction=0;
  order=1;
 
  ox=oy=0.0,scale=1.0;
@@ -729,13 +887,13 @@ int main(int argc,char *argv[])
 	"-n|--iterations:%d",&niter,
 	"-r|--sigma|--rejection|--rejection-level:%g",&rejlevel,
 	"-w|--weight:%s",&weightparam,
-	"--wcs:%s",&wcsparam,
+	"--wcs|--prj|--proj|--project|--projection:%s",&projection_param,
 	"-a|--order:%d",&order,
 	"-f|--offset:%g,%g",&ox,&oy,
 	"--scale:%g",&scale,
 	"--colr|--col-ref|--col-xy:%d,%d",&col.refx,&col.refy,
-	"--col-radec:" SNf(1) "%d,%d",&wcsconvert,&col.refx,&col.refy,
-	"--col-pixel:" SNf(2) "%d,%d",&wcsconvert,&col.refx,&col.refy,
+	"--col-radec:" SNf(1) "%d,%d",&projection_direction,&col.refx,&col.refy,
+	"--col-pixel:" SNf(2) "%d,%d",&projection_direction,&col.refx,&col.refy,
 	"--coli|--col-fit:%s",&colfitstr,
 	"--colo|--col-out:%s",&coloutstr,
 	"--col-sdk:%s",&colsdkstr,
@@ -769,7 +927,7 @@ int main(int argc,char *argv[])
 	return(0);
   }
 
- if ( intransfile == NULL && outtransfile == NULL && wcsparam == NULL )
+ if ( intransfile == NULL && outtransfile == NULL && projection_param == NULL )
   {	fprint_error("invalid combination of command line arguments");
 	return(1);
   }
@@ -799,7 +957,7 @@ int main(int argc,char *argv[])
 
  if ( colfitstr != NULL )
   {	free(col.fitc);
-	i=parse_number_list(colfitstr,&col.nval,&col.fitc);
+	i=grtrans_parse_number_list(colfitstr,&col.nval,&col.fitc);
 	if ( i || col.nval<1 )	
 	 {	fprint_error("invalid column specifications");
 		return(1);
@@ -812,39 +970,107 @@ int main(int argc,char *argv[])
 		 }
 	 }
   }
- if ( wcsparam != NULL )
-  {	int	is_deg,is_rad;
-	wcs=&wcs_data;
+ if ( projection_param != NULL )
+  {	int	is_deg,is_rad,w;
+	double	radscale,degscale;
+
+	wcs=&wcs_data_static;
 	wcs->init.ra0=0.0;
 	wcs->init.de0=0.0;
-	wcs->init.type=WCS_TAN;
+	wcs->init.roll0=0.0;
+	wcs->init.max_distance=0.0;
+	wcs->init.qr0=0.0;
+	wcs->init.qi0=0.0;
+	wcs->init.qj0=0.0;
+	wcs->init.qk0=0.0;
+	wcs->init.type=PROJECTION_TAN;
 	wcs->init.order=1;
 	wcs->init.zfactor=M_R2D;	/* 180 \over \pi */
+	wcs->init.distort.brown_conrady_radial_ncoeff=0;
+	wcs->init.distort.brown_conrady_radial_coeffs[0]=0.0;
+	wcs->init.distort.brown_conrady_radial_coeffs[1]=0.0;
+	wcs->init.distort.brown_conrady_radial_coeffs[2]=0.0;
+	wcs->init.distort.brown_conrady_radial_coeffs[3]=0.0;
+
+	wcs->init.distort.brown_conrady_tangential_ncoeff=0;
+	wcs->init.distort.brown_conrady_tangential_coeffs[0]=0.0;
+	wcs->init.distort.brown_conrady_tangential_coeffs[1]=0.0;
+	wcs->init.distort.brown_conrady_tangential_coeffs[2]=0.0;
+	wcs->init.distort.brown_conrady_tangential_coeffs[3]=0.0;
+
 	is_deg=is_rad=0;
-	i=scanpar(wcsparam,SCANPAR_DEFAULT,
+	radscale=degscale=0.0;
+
+	i=scanpar(projection_param,SCANPAR_DEFAULT,
 		"ra:%g" ,&wcs->init.ra0,
 		"dec:%g",&wcs->init.de0,
-		"tan:" SNf(WCS_TAN),&wcs->init.type,
-		"sin:" SNf(WCS_SIN),&wcs->init.type,
-		"arc:" SNf(WCS_ARC),&wcs->init.type,
+		"roll:%g",&wcs->init.roll0,
+		"max:%g",&wcs->init.max_distance,
+		"qr:%g",&wcs->init.qr0,
+		"qi:%g",&wcs->init.qi0,
+		"qj:%g",&wcs->init.qj0,
+		"qk:%g",&wcs->init.qk0,
+		"tan:" SNf(PROJECTION_TAN),&wcs->init.type,
+		"sin:" SNf(PROJECTION_SIN),&wcs->init.type,
+		"arc:" SNf(PROJECTION_ARC),&wcs->init.type,
 		"order:%d",&wcs->init.order,
-		"scale:%g",&wcs->init.zfactor,
+		"scale:%g",&radscale,
+		"radscale:%g",&radscale,
+		"degscale:%g",&degscale,
+		"k1:%g",&wcs->init.distort.brown_conrady_radial_coeffs[0],
+		"k2:%g",&wcs->init.distort.brown_conrady_radial_coeffs[1],
+		"k3:%g",&wcs->init.distort.brown_conrady_radial_coeffs[2],
+		"k4:%g",&wcs->init.distort.brown_conrady_radial_coeffs[3],
+		"p1:%g",&wcs->init.distort.brown_conrady_tangential_coeffs[0],
+		"p2:%g",&wcs->init.distort.brown_conrady_tangential_coeffs[1],
+		"p3:%g",&wcs->init.distort.brown_conrady_tangential_coeffs[2],
+		"p4:%g",&wcs->init.distort.brown_conrady_tangential_coeffs[3],
 		"deg|degree|degrees:%f",&is_deg,
 		"rad|radian|radians:%f",&is_rad,
 		NULL);
 	if ( i )	
-	 {	fprint_error("invalid WCS paramter in '%s'",wcsparam);
+	 {	fprint_error("invalid projection paramter in '%s'",projection_param);
 		return(1);
 	 }
-	if ( is_deg )	wcs->init.zfactor=M_R2D;
-	if ( is_rad )	wcs->init.zfactor=1.0;
+	else if ( radscale<0.0 || degscale<0.0 )
+	 {	fprint_error("invalid (negative) scale value");
+		return(1);
+	 }
+	w=0;
+	if ( 0.0<radscale )	w++;
+	if ( 0.0<degscale )	w++;
+	if ( is_rad )		w++;
+	if ( is_deg )		w++;
+	if ( 2<=w )
+	 {	fprint_error("there is a scale ambiguity in the projection type");
+		return(1);
+	 }
+
+	else if ( 0.0<radscale )	wcs->init.zfactor=radscale;
+	else if ( 0.0<degscale )	wcs->init.zfactor=M_R2D*degscale;
+	else if ( is_rad )		wcs->init.zfactor=1.0;
+	else /* if ( is_deg ) */	wcs->init.zfactor=M_R2D;
+
+	for ( i=3 ; 0<=i ; i-- )
+	 {	if ( wcs->init.distort.brown_conrady_radial_coeffs[i] != 0.0 )
+		 {	wcs->init.distort.brown_conrady_radial_ncoeff=i+1;
+			break;
+		 }
+	 }
+
+	for ( i=3 ; 0<=i ; i-- )
+	 {	if ( wcs->init.distort.brown_conrady_tangential_coeffs[i] != 0.0 )
+		 {	wcs->init.distort.brown_conrady_tangential_ncoeff=i+1;
+			break;
+		 }
+	 }
   }
  else
 	wcs=NULL;
 		
- if ( intransfile != NULL || wcsconvert )
+ if ( intransfile != NULL || projection_direction )
   {	if ( coloutstr != NULL )
-	 {	i=parse_number_list(coloutstr,&col.nval,&col.outc);
+	 {	i=grtrans_parse_number_list(coloutstr,&col.nval,&col.outc);
 		if ( i || col.nval<1 )	
 		 {	fprint_error("invalid output column specification");
 			return(1);
@@ -939,10 +1165,10 @@ int main(int argc,char *argv[])
   }
 
  else if ( outtransfile != NULL )
-  {	double	**vfits;
-	int	nval,nrej;
-	double	fd,dd,s,sd,sdd,sig,w;
-	fpoint	*wf;
+  {	double		**vfits;
+	int		nval,nrej;
+	double		fd,dd,s,sd,sdd,sig,w;
+	fitpoint	*wf;
 
 	nval=col.nval;
 
@@ -953,7 +1179,7 @@ int main(int argc,char *argv[])
 		return(1);
 	 }
 
-	read_data_points(fr,&col,&fps,&nfp);
+	grtrans_read_data_points(fr,&col,&fps,&nfp);
 	fclose(fr);
 
 	if ( order<0 )		
@@ -1058,7 +1284,7 @@ int main(int argc,char *argv[])
   }
 
  else if ( intransfile != NULL )
-  {	char	*rbuff,**cmd,*nstr,**npnt,*cpnt,outformat[8];
+  {	char	*rbuff,**cmd,*nstr,**npnt,*cpnt,outformat[32];
 	int	n,nval,mxc;
 	double	x,y,nw,is,id,ik,nx,ny;
 	double	*jxx,*jxy,*jyx,*jyy;
@@ -1225,13 +1451,16 @@ int main(int argc,char *argv[])
 
  else if ( wcs != NULL )
   {	
-	if ( wcsconvert )	/* wcs: ra,dec -> x,y or x,y -> ra,dec */
-	 {	double	ra,dec;
-		char	*rbuff,**cmd,*nstr,**npnt,*cpnt,outformat[8];
+	if ( projection_direction )	/* wcs: ra,dec -> x,y or x,y -> ra,dec */
+	 {	double	ra,dec,zmin;
+		char	*rbuff,**cmd,*nstr,**npnt,*cpnt,outformat[32];
 		int	n,nval,mxc;
 		double	x,y,z;
 	
-		wcs_get_projection_matrix(wcs->init.ra0,wcs->init.de0,wcs->init.mproj);
+		if ( wcs->init.qr0 != 0.0 || wcs->init.qi0 != 0.0 || wcs->init.qj0 != 0.0 || wcs->init.qk0 != 0.0 )
+			projection_get_matrix_quat(wcs->init.qr0,wcs->init.qi0,wcs->init.qj0,wcs->init.qk0,wcs->init.mproj);
+		else
+			projection_get_matrix_rdr(wcs->init.ra0,wcs->init.de0,wcs->init.roll0,wcs->init.mproj);
 
 		if ( infile==NULL )	fr=stdin;
 		else			fr=fopenread(infile);
@@ -1259,13 +1488,22 @@ int main(int argc,char *argv[])
 
 		rbuff=NULL;cmd=NULL;
 		sprintf(outformat,"%%%ds ",padsize);
+
+		if ( 0.0<wcs->init.max_distance )
+			zmin=-cos(wcs->init.max_distance*M_PI/180.0);
+		else
+			zmin=1.1;
+
 		while ( ! feof(fr) )
-		 {	if ( rbuff != NULL )	free(rbuff);
+		 {	
+			if ( rbuff != NULL )	free(rbuff);
 			if ( cmd != NULL )	{ free(cmd);cmd=NULL; }
+
 			rbuff=freadline(fr);
 			if ( rbuff==NULL )	break;
 			remove_newlines_and_comments(rbuff);
 			cmd=tokenize_spaces_dyn(rbuff);
+
 			if ( cmd==NULL )	continue;
 			for ( n=0 ; cmd[n] != NULL ; )	n++;
 			if ( n<=col.refx || n<=col.refy )	continue;
@@ -1273,22 +1511,24 @@ int main(int argc,char *argv[])
 			if ( sscanf(cmd[col.refy],"%lg",&y)<1 )	continue;
 			if ( ! isfinite(x) || ! isfinite(y) )	continue;
 
-			if ( ! ( wcsconvert-1 ) )	/* ra,dec -> x,y */
+			if ( ! ( projection_direction-1 ) )	/* ra,dec -> x,y */
 			 {	ra=x,dec=y;
-				wcs_get_projected_coords_matrix(wcs->init.mproj,ra,dec,&x,&y,&z);
-				wcs_project_distort(wcs->init.type,&x,&y,&z);
-				/* note '-' sign here: */
+				projection_do_matrix_coord(wcs->init.mproj,ra,dec,&x,&y,&z);
+				if ( zmin < z )
+					continue;
+				projection_do_distortion(wcs->init.type,&wcs->init.distort,&x,&y,&z);
+				/* note the '-' sign here: */
 				sprintf(npnt[0],"%15.9g",-x*wcs->init.zfactor);
 				sprintf(npnt[1],"%15.9g",+y*wcs->init.zfactor);
 			 }
 			else				/* x,y -> ra,dec */
-			 {	/* note '-' sign here: */
+			 {	/* note the '-' sign here: */
 				x=-x/wcs->init.zfactor;
 				y=+y/wcs->init.zfactor;
 				z=0.0;
-				wcs_invert_project_distort(wcs->init.type,&x,&y,&z);
 				ra=dec=0.0;
-				wcs_invert_projected_coords_matrix(wcs->init.mproj,x,y,&ra,&dec);
+				projection_do_inverse_distortion(wcs->init.type,&wcs->init.distort,&x,&y,&z);
+				projection_do_inverse_matrix_coord(wcs->init.mproj,x,y,&ra,&dec);
 				sprintf(npnt[0],"%15.9g",ra);
 				sprintf(npnt[1],"%15.9g",dec);
 			 }
@@ -1326,12 +1566,12 @@ int main(int argc,char *argv[])
 		 {	fprint_error("unable to open input data file '%s'",infile);
 			return(1);
 		 }
-		read_data_points(fr,&col,&fps,&nfp);
+		grtrans_read_data_points(fr,&col,&fps,&nfp);
 		fclose(fr);
-		fit_wcs(fps,nfp,&wcs->init,wcs);
+		grtrans_wcs_perform_fit(fps,nfp,&wcs->init,wcs);
 		fw=stdout;
-		dump_wcs_headers(fw,wcs);
-		free_wcs(wcs);
+		grtrans_wcs_dump_headers(fw,wcs);
+		grtrans_wcs_data_free(wcs);
 	 }
   }
 
