@@ -636,6 +636,8 @@ psnlfit psnlfit_list_builtin_aa_functions[]=
 
 #define		F_HJD		110		/* hjd(), helocentric JD     */
 #define		F_BJD		111		/* bjd(), barycentric JD     */
+#define		F_DHJD		112		/* dhjd(), helocentric JD     */
+#define		F_DBJD		113		/* dbjd(), barycentric JD     */
 
 static int funct_f_eoq(double *s)
  {	s-=3;
@@ -751,6 +753,7 @@ static int funct_f_ntiq_dg2(double *s)
 	return(0);
  }
 
+
 static int funct_f_hjd(double *s)
  {	s-=3;
 	*s=get_hjd(*s,*(s+1),*(s+2));
@@ -763,6 +766,18 @@ static int funct_f_bjd(double *s)
 	return(0);
  }
 
+static int funct_f_dhjd(double *s)
+ {	s-=3;
+	*s=get_diff_hjd(*s,*(s+1),*(s+2));
+	return(0);
+ }
+
+static int funct_f_dbjd(double *s)
+ {	s-=3;
+	*s=get_diff_bjd(*s,*(s+1),*(s+2));
+	return(0);
+ }
+
 static int funct_f_elliptic_k(double *s)
 {
 	s-=1;
@@ -803,6 +818,8 @@ static psnfunctinfo	pfi_ntiu	= { "normalized transit intensity - uniform flux di
 static psnfunctinfo	pfi_ntiq	= { "normalized transit intensity - quadratic limb darkening assumption (arguments: fracional radius, normalized distance, the two limb darkening coefficients)" };
 static psnfunctinfo	pfi_hjd		= { "heliocentric Julian date (arguments: julian day, RA and DEC, in degrees)" };
 static psnfunctinfo	pfi_bjd		= { "barycentric Julian date (arguments: julian day, RA and DEC, in degrees)" };
+static psnfunctinfo	pfi_dhjd	= { "difference between HJD and JD, i.e. hjd()=jd+dhjd(jd,.,.) (arguments: julian day, RA and DEC, in degrees)" };
+static psnfunctinfo	pfi_dbjd	= { "difference between BJD and JD, i.e. bjd()=jd+dbjd(jd,.,.) (arguments: julian day, RA and DEC, in degrees)" };
 static psnfunctinfo	pfi_ell_k	= { "complete elliptic integral of the first kind" };
 static psnfunctinfo	pfi_ell_e	= { "complete elliptic integral of the second kind" };
 static psnfunctinfo	pfi_ell_pi	= { "complete elliptic integral of the third kind" };
@@ -821,6 +838,8 @@ psnlfit psnlfit_list_builtin_xfuncts[]=
   { "ntiq_dg2"  , T_FN, F_NTIQ_DG2, 0        , 4,  0, 0          , funct_f_ntiq_dg2   , NULL            , "ntiq_dg2(#1,#2,#3,#4)", 0, 0   , NULL  	},
   { "hjd"       , T_FN, F_HJD     , 0        , 3,  0, 0          , funct_f_hjd        , NULL            , "hjd(#1,#2,#3)"        , 0, 0   , &pfi_hjd	},
   { "bjd"       , T_FN, F_BJD     , 0        , 3,  0, 0          , funct_f_bjd        , NULL            , "bjd(#1,#2,#3)"        , 0, 0   , &pfi_bjd	},
+  { "dhjd"      , T_FN, F_DHJD    , 0        , 3,  0, 0          , funct_f_dhjd       , NULL            , "dhjd(#1,#2,#3)"       , 0, 0   , &pfi_dhjd	},
+  { "dbjd"      , T_FN, F_DBJD    , 0        , 3,  0, 0          , funct_f_dbjd       , NULL            , "dbjd(#1,#2,#3)"       , 0, 0   , &pfi_dbjd	},
   { "ellipticK" , T_FN, F_ELL_K   , 0        , 1,  0, 0          , funct_f_elliptic_k , NULL            , "ellipticK(#1)"        , 0, 0   , &pfi_ell_k	},
   { "ellipticE" , T_FN, F_ELL_E   , 0        , 1,  0, 0          , funct_f_elliptic_e , NULL            , "ellipticE(#1)"        , 0, 0   , &pfi_ell_e	},
   { "ellipticPi", T_FN, F_ELL_PI  , 0        , 2,  0, 0          , funct_f_elliptic_pi, NULL            , "ellipticPi(#1)"       , 0, 0   , &pfi_ell_pi	},
