@@ -28,7 +28,7 @@ int fprint_lfit_usage(FILE *fw)
 {
  fprintf(fw,
 "Usage:\tlfit [-h|--help|--long-help|--wiki-help] [--version[-short]]\n"
-"\t[--examples] [--function-list]\n"
+"\t[--examples] [--function-list|--function-wiki]\n"
 "\t[-V|--verbose] [--quiet] [-o|--output <output>] \n");
  fprintf(fw,
 "Common parameters and arguments for fitting and/or regression analysis:\n"
@@ -110,6 +110,9 @@ longhelp_entry lfit_long_help[]=
  { "--functions, --list-functions, --function-list", 
 	"Lists the available arithmetic operations and built-in functions "
 	"supported by the program." },
+ { "--wiki-functions, --functions-wiki", 
+	"Lists the available arithmetic operations and built-in functions "
+	"supported by the program in Mediawiki format." },
  { "--examples",
 	"Prints some very basic examples for the program invocation." },
 
