@@ -625,11 +625,11 @@ void gauss_2d_wdev_exact_init(void *xpnt,double *a,double *yy,double *dyda,void
 			cfdy[I13]=M12*s*cf[I12];
 			cfdy[I04]=M03*s*cf[I03];
 			cfdy[I50]=0.0;
-			cfdx[I41]=M40*s*cf[I40];
-			cfdx[I32]=M31*s*cf[I31];
-			cfdx[I23]=M22*s*cf[I22];
-			cfdx[I14]=M13*s*cf[I13];
-			cfdx[I05]=M04*s*cf[I04];
+			cfdy[I41]=M40*s*cf[I40];
+			cfdy[I32]=M31*s*cf[I31];
+			cfdy[I23]=M22*s*cf[I22];
+			cfdy[I14]=M13*s*cf[I13];
+			cfdy[I05]=M04*s*cf[I04];
 			for ( i=0,w=0.0,k=0 ; i<=5 ; i++ )
 			 {	for ( j=0 ; j<=i ; j++,k++ )
 					w+=cfdy[k]*ix[i-j]*iy[j];
