@@ -36,4 +36,39 @@ double median(double *data,int n)
 }
 
 /*****************************************************************************/
-          
+
+double mean(double *arr,int n)
+{
+ int    i;
+ double r;
+ if ( arr==NULL || n<=0 )
+        return(0.0);
+ r=0.0;
+ for ( i=0 ; i<n ; i++ )
+  {     r+=arr[i];              }
+ return(r/(double)n);
+}
+
+/*****************************************************************************/
+                                   
+double truncated_mean(double *data,int n,int t)
+{
+ double m;
+ int	i,r;
+
+ if ( data==NULL || n<=0 )
+	return(0.0);
+
+ qsort((void *)data,n,sizeof(double),median_compare);
+
+ m=0;
+ r=0; 
+ for ( i=t; i<n-t; i++ )
+  {	m+=data[i];
+	r++;
+  }
+ if ( 0<r )	m=m/(double)r;
+
+ return(m);
+}
+
